import os

from ._volume import is_volume_archive_by_filename
from ..file import guess_filetype


def is_archive_by_filename(filename: str) -> bool:
    """通过文件名判断是否为压缩文件
    :param filename: str，文件名（包含文件扩展名）
    :return: bool，是否为压缩包
    """
    _archive_file_extension = ['zip', 'rar', '7z', 'tar', 'gz', 'xz', 'iso']

    #  提取文件后缀名（不带.），判断一般的压缩文件
    file_extension = os.path.splitext(filename)[1].strip().strip('.').strip()
    if file_extension.lower() in _archive_file_extension:
        return True

    # 检查是否为分卷压缩文件
    if is_volume_archive_by_filename(filename):
        return True

    return False


def is_archive(filepath: str) -> bool:
    """通过文件头判断是否为压缩文件
    :param filepath: str，文件路径
    :return: bool，是否为压缩包
    """
    _archive_file_extension = ['zip', 'tar', 'rar', 'gz', '7z', 'xz']  # filetype库支持的压缩文件后缀名
    guess_type = guess_filetype(filepath)
    if guess_type and guess_type in _archive_file_extension:
        return True

    return False



import math
from typing import Tuple

import pyautogui

# 默认参数
_DEFAULT_MOVE_DURATION: float = 0.25  # 移动所需时间
_DEFAULT_BUTTON: str = 'left'  # 默认鼠标按键
_DEFAULT_CLICKS: int = 1  # 点击次数
_DEFAULT_INTERVAL: float = 0.1  # 间隔时间
_DEFAULT_MOVE_DIRECTION: int = 0  # 移动方向
_DEFAULT_MOVE_DISTANCE: int = 100  # 移动距离
_DEFAULT_SCROLL_DISTANCE: int = 100  # 滚动滚动距离
_MAX_X, _MAX_Y = pyautogui.size()  # x,y坐标值的最大值限制（屏幕大小）


def _check_position(x: int, y: int):
    """检查xy轴，0,0->1,1"""
    if x == 0 and y == 0:  # 0,0替换为1,1，防止提前终止
        return 1, 1
    else:
        return x, y


def _check_button(button: str):
    """检查按键，统一为pyautogui支持的格式"""
    if button.lower() in ['left', '左键']:
        return 'left'
    elif button.lower() in ['right', '右键']:
        return 'right'
    elif button.lower() in ['middle', '中键']:
        return 'middle'
    else:
        raise Exception(f'非法参数：{button}')


def get_position() -> Tuple[int, int]:
    """获取当前鼠标的位置坐标
    :return: x轴坐标，y轴坐标"""
    x, y = pyautogui.position()

    return x, y


def move_to_position(x: int, y: int, move_duration: float = _DEFAULT_MOVE_DURATION):
    """移动鼠标至指定坐标轴
    :param x: int，x轴坐标
    :param y: int，y轴坐标
    :param move_duration: float，移动所需时间（秒）"""
    x, y = _check_position(x, y)  # 检查坐标

    pyautogui.moveTo(x, y, duration=move_duration)

    return x, y


def drag_to_position(x: int, y: int, button: str = _DEFAULT_BUTTON,
                     move_duration: float = _DEFAULT_MOVE_DURATION):
    """按下鼠标键后拖拽至指定坐标轴
    :param x: int，x轴坐标
    :param y: int，y轴坐标
    :param button: str，鼠标按键，left/左键/right/右键/middle/中键
    :param move_duration: float，移动所需时间（秒）
    """
    x, y = _check_position(x, y)  # 检查坐标
    button = _check_button(button)  # 检查按键
    pyautogui.dragTo(x, y, button=button, duration=move_duration)

    return x, y


def move_relative(move_direction: int = _DEFAULT_MOVE_DIRECTION,
                  move_distance: int = _DEFAULT_MOVE_DISTANCE, move_duration: float = _DEFAULT_MOVE_DURATION):
    """以鼠标当前位置为基点，相对移动鼠标
    :param move_direction: int，移动角度（水平向左为0°，垂直向下为90°，水平向右为180°或-180°，垂直向上为270°或-90°）
    :param move_distance: int，移动距离
    :param move_duration: float，移动所需时间（秒）
    """
    # 计算目标位置的坐标轴
    x, y = pyautogui.position()
    angle_radians = math.radians(move_direction)  # 转弧度
    x = x + move_distance * math.cos(angle_radians)
    y = y + move_distance * math.sin(angle_radians)

    # 检查xy
    if x < 0:
        x = 1
    if x > _MAX_X:
        x = _MAX_X

    if y < 0:
        y = 1
    if y > _MAX_X:
        y = _MAX_X

    x, y = _check_position(x, y)  # 检查坐标

    pyautogui.moveTo(x, y, duration=move_duration)

    return x, y


def click(button: str = _DEFAULT_BUTTON, clicks: int = _DEFAULT_CLICKS,
          interval: float = _DEFAULT_INTERVAL):
    """点击鼠标按键
    :param button: str，鼠标按键，left/right/middle
    :param clicks: int，点击次数
    :param interval: float，两次点击之间的间隔时间（秒）"""
    button = _check_button(button)  # 检查按键
    pyautogui.click(button=button, clicks=clicks, interval=interval)

    return button


def press_down(button: str = _DEFAULT_BUTTON):
    """按下鼠标按键
    :param button: str，鼠标按键，left/right/middle"""
    button = _check_button(button)  # 检查按键
    pyautogui.mouseDown(button=button)

    return button


def release(button: str = _DEFAULT_BUTTON):
    """释放鼠标按键
    :param button: str，鼠标按键，left/right/middle"""
    button = _check_button(button)  # 检查按键
    pyautogui.mouseUp(button=button)

    return button


def scroll_wheel(distance: int = _DEFAULT_SCROLL_DISTANCE):
    """滚动鼠标滚轮
    :param distance: int，滚动距离，正数向上滚动，负数向下滚动"""
    pyautogui.scroll(clicks=distance)

    return distance



from PySide6.QtCore import *
from PySide6.QtGui import *
from PySide6.QtWidgets import *


class TabelWidgetHiddenOverLengthText(QTableWidget):
    """文本框控件，自动隐藏长文本（abcd->a...）
    备注：利用tableWidget的文本单元格自动隐藏长文本的特性"""

    def __init__(self, parent=None):
        super().__init__(parent)
        self.setFrameShape(QFrame.NoFrame)
        # 设置列宽度为自动适应控件大小
        self.horizontalHeader().setStretchLastSection(True)
        self.horizontalHeader().setSectionResizeMode(QHeaderView.ResizeMode.Stretch)
        # 隐藏行列
        self.verticalHeader().setVisible(False)
        self.horizontalHeader().setVisible(False)
        # 设置为单行单列
        self.setColumnCount(1)
        self.insertRow(0)
        # 固定控件高度、单元格行高
        self.setFixedHeight(18)
        self.setRowHeight(0, 16)
        # 设置文本单元格
        self.item_filename = QTableWidgetItem('')
        self.setItem(0, 0, self.item_filename)
        # 禁止编辑单元格
        self.item_filename.setFlags(self.item_filename.flags() & ~Qt.ItemIsEditable)

    def set_text(self, text: str):
        """设置文本"""
        self.item_filename.setText(text)

    def set_tooltip(self, tool_tip: str):
        """设置控件提示"""
        self.item_filename.setToolTip(tool_tip)

    def set_height(self, height: int):
        """设置控件高度
        :param height: int，高度"""
        self.setFixedHeight(height + 2)  # 控件高度
        self.setRowHeight(0, height)  # 单元格行高



from PySide6.QtCore import *
from PySide6.QtGui import *
from PySide6.QtWidgets import *


class StyledItemDelegateImage(QStyledItemDelegate):
    """项目视图委托QStyledItemDelegate，在QStandardItem中显示自适应大小的图像
    注意：图像路径需在保存在QStandardItem的UserRole属性中"""

    def __init__(self, parent=None):
        super().__init__(parent)

    def paint(self, painter, option, index):
        # 获取QStandardItem中的图片数据
        pixmap = index.data(Qt.UserRole)
        if not pixmap:
            raise Exception('错误，图像路径需在保存在QStandardItem的UserRole属性中')

        # 创建绘制工具
        item_rect = option.rect
        painter.setRenderHint(QPainter.Antialiasing)
        painter.setBrush(Qt.NoBrush)

        # 缩放图片以适应QStandardItem
        scaled_pixmap = pixmap.scaled(QSize(item_rect.width(), item_rect.height()),
                                      Qt.KeepAspectRatio, Qt.SmoothTransformation)

        # 在QStandardItem上绘制图片
        painter.drawPixmap(item_rect.x() + (item_rect.width() - scaled_pixmap.width()) / 2,
                           item_rect.y() + (item_rect.height() - scaled_pixmap.height()) / 2,
                           scaled_pixmap.width(), scaled_pixmap.height(), scaled_pixmap)



import pickle
import socket


def send_data_to_socket(data, host: str, port: str):
    """向本地端口传递数据（使用socket）
    :param data: 任意类型的数据
    :param host: str，主机地址
    :param port: str，端口"""
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    server_address = (host, port)
    sock.connect(server_address)

    try:
        # 发送数据
        serialized_data = pickle.dumps(data)  # 用pickle序列化，传递更多类型的数据
        sock.sendall(serialized_data)
    finally:
        # 关闭连接
        sock.close()



from PySide6.QtCore import *
from PySide6.QtGui import *
from PySide6.QtWidgets import *


class ToolButtonRightClick(QToolButton):
    """支持右键点击信号的QToolButton"""
    rightClicked = Signal()

    def __init__(self, parent=None):
        super().__init__(parent)
        self.setContextMenuPolicy(Qt.ContextMenuPolicy.CustomContextMenu)
        self.setMouseTracking(True)
        self.customContextMenuRequested.connect(self._emit_signal)

    def _emit_signal(self):
        self.rightClicked.emit()



import time


def get_current_time(_format: str = '%Y-%m-%d %H:%M:%S') -> str:
    """获取当前时间的标准化格式str
    :param _format: str，自定义时间格式
    :return: str，时间格式表示的字符串"
    """
    return time.strftime(_format, time.localtime())


def convert_time(runtime: float):
    """将一个时间差转换为时分秒的字符串"""
    hours = int(runtime // 3600)
    minutes = int((runtime % 3600) // 60)
    seconds = int(runtime % 60)
    time_str = f'{hours}:{minutes:02d}:{seconds:02d}'

    return time_str



from PySide6.QtGui import *
from PySide6.QtWidgets import *

from ._function import set_transparent_background


class DialogGifPlayer(QDialog):
    """播放GIF动画的QDialog"""

    def __init__(self, parent=None):
        super().__init__(parent)
        set_transparent_background(self)

        # 添加label
        self.label_gif = QLabel('GIF PLAYER')
        self.layout = QVBoxLayout(self)
        self.layout.addWidget(self.label_gif)

        # 添加动画对象
        self.gif = None

    def set_gif(self, gif: str):
        """设置gif
        :param gif: str，gif文件路径"""
        self.gif = QMovie(gif)
        self.label_gif.setMovie(self.gif)

    def play(self):
        self.gif.start()
        self.show()

    def stop(self):
        self.gif.stop()
        self.close()



from PySide6.QtCore import *
from PySide6.QtGui import *
from PySide6.QtWidgets import *

from ._function import set_transparent_background


class TopTip(QMainWindow):
    """显示一个置顶的提示文本
    新增功能：淡入淡出动画"""

    def __init__(self, text: str):
        super().__init__()

        # 添加显示文字的子控件
        self.label_showed = None
        self._add_label(str(text))

        # 设置定时器
        self.timer_fade = QTimer(self)
        self.timer_fade.timeout.connect(self._fade_out)
        self.timer_fade.start(2000)  # 留存2秒

        # 设置淡入淡出动画
        self.animation = QPropertyAnimation()
        self._set_animation()

        # 设置透明属性
        set_transparent_background(self)
        # 设置置顶显示
        self.setWindowFlags(self.windowFlags() | Qt.WindowStaysOnTopHint)

        # 更新控件大小（需要在添加子控件后更新）
        self.setFixedSize(self.sizeHint())

        # 显示
        self._show_in_center()
        self.animation.start()

    def set_duration(self, duration: int):
        """设置提示文本留存时间"""
        self.timer_fade.start(duration)

    def _add_label(self, text: str):
        """添加子控件"""
        label = QLabel(str(text))
        label.setStyleSheet("font-size: 15pt; color: blue")
        self.setCentralWidget(label)

    def _show_in_center(self):
        """在屏幕中心显示"""
        screen = QGuiApplication.primaryScreen().availableGeometry()
        size = self.geometry()
        x = int((screen.width() - size.width()) / 2)
        y = int((screen.height() - size.height()) / 2)
        print(x, y)
        self.move(x, y)

        self.show()

    def _set_animation(self):
        """设置动画"""
        opacity_effect = QGraphicsOpacityEffect(self)
        self.setGraphicsEffect(opacity_effect)
        self.animation = QPropertyAnimation(opacity_effect, b"opacity")
        self.animation.setDuration(500)
        self.animation.setStartValue(0.0)
        self.animation.setEndValue(1.0)
        self.animation.setEasingCurve(QEasingCurve.InOutCubic)

    def _fade_out(self):
        """淡出效果"""
        self.animation.setStartValue(1.0)
        self.animation.setEndValue(0.0)
        self.animation.finished.connect(self.close)

        self.animation.start()



import imagehash
from PIL import Image

from ._similar import numpy_hash_to_str


def is_pure_color_image(image_path: str) -> bool:
    """是否为纯色图片
    :param image_path: str，图片路径"""
    # 考虑到库的大小，通过计算图片Hash值的方法来判断是否为纯色图片（不使用opencv库，该库太大）
    try:
        image_pil = Image.open(image_path)
        image_pil = image_pil.convert('L')  # 转灰度图
    except OSError:  # 如果图片损坏，会抛出异常OSError: image file is truncated (4 bytes not processed)
        return False

    dhash = imagehash.average_hash(image_pil, hash_size=16)
    hash_str = numpy_hash_to_str(dhash)

    if hash_str.count('0') == len(hash_str):
        return True
    else:
        return False



import sqlite3
from typing import Union


class SQLiteDB:
    """
    sqlite3 数据库方法简单封装
    :param file_db: str类型，数据库文件名
    """

    class ConditionList:
        """
        sqlite3条件列表，WHERE 语句的容器
        支持的逻辑运算符: =, >, <, LIKE, NOT...
        字符串值需要额外添加引号"防止报错
        """

        def __init__(self, conj: str = 'add'):
            """
            :param conj: str类型，'add' 表示且，'or' 表示或
            """
            super().__init__()
            self.conj = conj
            self.rules = []

        def add(self, *, column: str, logical_operator: str, value: any):
            """
            添加条件
            :param column: str类型，列名
            :param logical_operator: str类型，逻辑运算符: =, >, <, LIKE, NOT...
            :param value: 任意类型，列对应的值
            """
            rule = f'{column} {logical_operator} "{value}"'
            self.rules.append(rule)

        def get_sql_condition(self):
            """获取sqlite3条件语句"""
            sql_condition = ' WHERE ' + f' {self.conj.upper()} '.join(self.rules)
            return sql_condition

    class OrderDict(dict):
        """
        sqlite3排序字典
        key为列名，value为升序 'ASC'/ 降序 'DESC'
        """

        def __init__(self):
            super().__init__()

        def get_sql_order(self):
            """获取sqlite3排序语句"""
            sql_order = ', '.join([f'{key} {value}' for key, value in self.items()])
            sql_order += ' ORDER BY '

            return sql_order

    def __init__(self, file_db: str = None):
        # 规范文件名
        if file_db:
            if file_db[-3:].lower() != '.db':
                file_db = file_db + '.db'
        else:
            file_db = 'database.db'

        self._conn = sqlite3.connect(file_db)
        self._cur = self._conn.cursor()

    @staticmethod
    def _convert_columns_datatype(columns_datatype: dict, primary_key: str = None) -> dict:
        """
        转换列名-数据类型对应字典
        :param columns_datatype: dict类型，key为列名，value为对应的数据类型
        :param primary_key: str类型，数据表的主键，默认无
        :return: dict类型，原始字典转换数据类型后的字典
        """
        # python的数据类型与sqlite数据库类型对应关系
        convert_dict = {None: 'NULL', int: 'INTEGER', float: 'REAL', str: 'TEXT', bytes: 'BLOB'}

        convert_column_dict = {}
        for column, datatype in columns_datatype.items():
            convert_datatype = convert_dict[datatype]
            convert_column_dict[column] = convert_datatype

        # 插入主键文本
        if primary_key in convert_column_dict:
            convert_column_dict[primary_key] = f'{convert_column_dict[primary_key]} Primary KEY'

        return convert_column_dict

    @staticmethod
    def _get_sql_col_type(columns_datatype: dict):
        """获取列名-数据类型的sql字段"""
        sql_col_type = ', '.join([f"{column} {datatype}" for column, datatype in columns_datatype.items()])

        return sql_col_type

    def create_table(self, table_name: str, columns_datatype: dict, primary_key: str = None):
        """
        创建数据表（自动忽略重复表）
        :param table_name: str类型，数据表名称
        :param columns_datatype: dict类型，key为列名，value为对应的数据类型
        :param primary_key: str类型，数据表的主键，默认无
        """
        # 将列名与数据类型的格式转换为sqlite支持的格式
        columns_datatype_converted = self._convert_columns_datatype(columns_datatype, primary_key)
        sql_col_type = self._get_sql_col_type(columns_datatype_converted)

        # 执行语句，使用'IF NOT EXISTS' 防止存在同名表时报错
        sql = f"CREATE TABLE IF NOT EXISTS {table_name} ({sql_col_type})"
        print(f'创建数据表sql: {sql}')
        self._cur.execute(sql)
        self._conn.commit()

    def insert_record_single(self, table_name: str, columns_value: dict):
        """
        在数据表中插入单行数据（自动忽略重复值）
        :param table_name: str类型，需要插入数据的数据表名称
        :param columns_value: dict类型，key为列名，value为对应的值
        """
        # 将列名与值的格式转换为sqlite支持的格式
        sql_column = ', '.join([column for column in columns_value.keys()])
        # 对于值的处理，需要都转为字符串，并且引号用"（Windows文件名可以用'而不能用"，用"来定义字符串防止报错）
        sql_value = ", ".join([f'"{value}"' for value in columns_value.values()])

        # 执行语句，使用'OR IGNORE' 来避免插入重复数据
        sql = f'INSERT OR IGNORE INTO {table_name} ({sql_column}) VALUES ({sql_value})'
        print(f'插入记录sql: {sql}')
        self._cur.execute(sql)
        self._conn.commit()

    def update_record(self, table_name: str, columns_value: dict, condition_list: ConditionList = None):
        """
        在数据表中更新行项目数据
        :param table_name: str类型，需要插入数据的数据表名称
        :param columns_value: dict类型，key为列名，value为对应的值
        :param condition_list: 自定义ConditionList类型，筛选条件
        """
        # 将列名与值的格式转换为sqlite支持的格式
        # 引号用"（Windows文件名可以用'而不能用"，用"来定义字符串防止报错）
        sql_col_value = ', '.join([f'{column} = "{value}"' for column, value in columns_value.items()])

        # 组合完整语句
        sql = f"UPDATE {table_name} SET {sql_col_value}"
        if condition_list:
            sql += condition_list.get_sql_condition()
        print(f'更新记录sql: {sql}')
        self._cur.execute(sql)
        self._conn.commit()

    def delete_record(self, table_name: str, condition_list: ConditionList = None):
        """
        在数据表中删除行项目数据
        :param table_name: str类型，需要插入数据的数据表名称
        :param condition_list: 自定义ConditionList类型，筛选条件
        """
        sql = f"DELETE FROM {table_name}"
        if condition_list:
            sql += condition_list.get_sql_condition()
        print(f'删除记录sql: {sql}')
        self._cur.execute(sql)
        self._conn.commit()

    def select_record(self, table_name: str, columns: Union[str, list] = '*', return_format: str = 'dict',
                      condition_list: ConditionList = None, order_dict: OrderDict = None) -> list:
        """
        在数据表查询行项目数据
        :param table_name: str类型，需要插入数据的数据表名称
        :param columns: str或list类型，需要查询的列名，默认为查询全部列
        :param return_format: str类型，返回数据的类型，tuple或dict
        :param condition_list: 自定义ConditionList类型，筛选条件
        :param order_dict: 自定义OrderDict类型，排序条件
        """
        # 组合语句
        if type(columns) is str:
            columns = [columns]
        sql = f'SELECT {", ".join(columns)} FROM {table_name}'

        if condition_list:
            sql += condition_list.get_sql_condition()

        if order_dict:
            sql += order_dict.get_sql_order()
        print(f'查询记录sql: {sql}')

        # 取得返回结果
        data = self._cur.execute(sql)
        columns_name = [i[0] for i in data.description]  # 列名
        original_data = data.fetchall()  # 原始格式为[(每行列值组成的元组), (a1_1,a1_2,a1_3), ...]

        # 调整返回结果的格式
        if return_format == 'tuple':
            return original_data
        else:
            return [dict(zip(columns_name, row)) for row in original_data]  # [{列名:列值...}...]

    def insert_column(self, table_name: str, columns_datatype: dict):
        """
        在数据表中插入新的列
        :param table_name: str类型，数据表名称
        :param columns_datatype: dict类型，key为列名，value为对应的数据类型
        """
        # 将列名与数据类型的格式转换为sqlite支持的格式
        columns_datatype_converted = self._convert_columns_datatype(columns_datatype)
        sql_col_type = self._get_sql_col_type(columns_datatype_converted)

        # 组合语句
        sql = f"ALTER TABLE {table_name} ADD COLUMN {sql_col_type}"
        print(f'插入新列sql: {sql}')
        self._cur.execute(sql)
        self._conn.commit()

    def copy_table(self, original_table_names: str, new_table_name: str):
        """
        复制数据表
        :param original_table_names: str类型，被复制的数据表名称
        :param new_table_name: str类型，新的数据表名称
        """
        # 复制数据表的列名
        sql_frame = f"CREATE TABLE {new_table_name} AS SELECT * FROM {original_table_names} WHERE 0"
        self._cur.execute(sql_frame)
        # 复制数据表的行项目
        sql_record = f"INSERT INTO {new_table_name} SELECT * FROM {original_table_names}"
        self._cur.execute(sql_record)

        self._conn.commit()

    def is_table_exist(self, table_name: str):
        """
        检查数据库中是否存在某个数据表
        :param table_name: str类型，数据表名称
        """
        sql = f"SELECT name FROM sqlite_master WHERE type='table' AND name='{table_name}'"
        self._cur.execute(sql)
        if self._cur.fetchone() is not None:
            return True
        else:
            return False

    def get_all_table_name(self):
        """返回数据库中的所有的数据表名称"""
        sql = f"SELECT name FROM sqlite_master WHERE type='table'"
        self._cur.execute(sql)

        return self._cur.fetchone()

    def count_table(self, table_name: str) -> int:
        """
        查询数据表中的行项目个数
        :param table_name: str类型，数据表名称
        """
        sql = f"SELECT COUNT(*) FROM {table_name};"
        self._cur.execute(sql)
        count = self._cur.fetchone()[0]

        return count

    def close(self):
        """关闭数据库连接"""
        self._conn.close()



# Pyside6的参考示例
from ._button import *
from ._dialog import *
from ._function import *
from ._label import *
from ._lineEdit import *
from ._listWidget import *
from ._mainWindow import *
from ._scrollArea import *
from ._slider import *
from ._styledItemDelegate import *
from ._tableWidget import *
from ._thread import *
from ._timer import *
from ._widget import *



import base64
import io

import cv2
import numpy
from PIL import Image


def bytes_to_numpy(bytes_image: bytes) -> numpy.ndarray:
    """将bytes图片对象转换为numpy图片对象
    :param bytes_image: bytes图片对象
    :return: numpy图片对象"""
    bytesio_image = io.BytesIO(bytes_image)  # 转为BytesIO对象
    pil_image = Image.open(bytesio_image)  # 转PIL.Image
    numpy_image = numpy.array(pil_image)  # 转NumPy数组

    return numpy_image


def numpy_to_bytes(numpy_image: numpy.ndarray) -> bytes:
    """将numpy图片对象转换为bytes图片对象
    :param numpy_image: numpy图片对象
    :return: bytes图片对象"""
    image = Image.fromarray(numpy_image)  # 换为PIL Image对象
    bytes_image = io.BytesIO()  # 转为BytesIO对象
    image.save(bytes_image, format=image.format)
    bytes_image = bytes_image.getvalue()

    return bytes_image


def resize_image_numpy(image: numpy.ndarray, width: int, height: int) -> numpy.ndarray:
    """缩放numpy图片对象至指定宽高
    :param image: numpy图片对象
    :param width: int，新的宽度
    :param height: int，新的高度"""
    image_ = cv2.resize(image, dsize=(width, height))
    return image_


def resize_image_numpy_ratio(image: numpy.ndarray, ratio: float) -> numpy.ndarray:
    """按比例缩放numpy图片对象
    :param image: numpy图片对象
    :param ratio: float，缩放比例（>0）"""
    width, height = image.size
    new_width = int(width * ratio)
    new_height = int(height * ratio)
    image_ = cv2.resize(image, dsize=(new_width, new_height))
    return image_


def rgb_to_gray_numpy(image: numpy.ndarray) -> numpy.ndarray:
    """将numpy图片对象转为灰度图"""
    image_ = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)
    return image_


def image_to_base64(image_path: str) -> str:
    """本地图片转为base64图片
    :param image_path: str，本地图片路径
    :return: bytes，base64字符串"""
    with open(image_path, "rb") as image_file:
        image_data = image_file.read()
        base64_str = base64.b64encode(image_data).decode('utf-8')
    return base64_str



"""
以Windows本地环境的排序规则对传入列表/传入路径的内部文件列表进行排序（主要用于排序中文）

列表排序实现方法：
1. 初始化本地环境
2. 使用冒泡排序，对比两个键的顺序，将其每个字符拆分，逐级对比
3. 返回排序后的列表

路径排序实现方法：
1. 遍历传入路径，按需提取文件或文件夹
2. 对路径中的每一个文件夹及其自身进行列表排序
3. 返回排序后的列表
"""

import locale
import os
import re
from typing import Union


def sort_list(_list: list, order_type: str = 'ASC') -> Union[list, SystemExit]:
    """排序列表
    :param _list: 需要排序的list
    :param order_type: 排序类型，'ASC' 升序或 'DESC' 降序，默认为升序
    :return: 排序后的list
    """
    # 初始化本地环境
    locale.setlocale(locale.LC_ALL, '')

    # 冒泡排序
    _list = _list.copy()
    n = len(_list)
    for i in range(n):
        for j in range(n - i - 1):
            if _is_reversal_two_key(_list[j], _list[j + 1]):
                _list[j], _list[j + 1] = _list[j + 1], _list[j]

    # 按升降序参数返回对应list
    if order_type.upper() == 'ASC':
        return _list
    elif order_type.upper() == 'DESC':
        return _list[::-1]
    else:
        raise Exception(f'排序类型错误：{order_type}')


def sort_path(_dirpath: str, order_type: str = 'ASC', filetype: str = 'both', walk_depth: int = 0) -> list:
    """排序指定文件夹中的下级文件/文件夹
    :param _dirpath: 需要排序的文件夹路径
    :param order_type: 排序类型，'ASC' 升序或 'DESC' 降序，默认为升序
    :param filetype: 需要排序的文件类型，'file' 文件或 'folder' 文件夹或 'both' 两者皆有，默认为两者皆有
    :param walk_depth: 遍历的层级深度，默认为0（排序所有下级层数的文件/文件夹）
    :return: 排序后的内部文件完整路径list
    """
    path_sorted = [_dirpath]  # 存放最终结果
    current_listdir_sorted_folder = [_dirpath]  # 当前层级中的文件夹
    current_listdir_sorted_folder_copy = current_listdir_sorted_folder.copy()  # 用于递归
    if walk_depth == 0:
        walk_depth = 100

    current_depth = 0
    while current_depth < walk_depth and current_listdir_sorted_folder:
        current_depth += 1
        for path in current_listdir_sorted_folder_copy:
            current_listdir_sorted = _sort_path_listdir(path, order_type=order_type, filetype=filetype)
            current_index = path_sorted.index(path)
            path_sorted[current_index + 1:current_index + 1] = current_listdir_sorted  # 利用切片插入列表元素（不能用insert）
            current_listdir_sorted_folder.remove(path)
            current_listdir_sorted_folder += [i for i in current_listdir_sorted if os.path.isdir(i)]
        current_listdir_sorted_folder_copy = current_listdir_sorted_folder.copy()

    # 删除一开始赋值的多余的项目
    path_sorted.remove(_dirpath)
    # 额外处理文件类型为file时的情况
    if filetype == 'file':
        path_sorted = [i for i in path_sorted if os.path.isfile(i)]

    return path_sorted


def _sort_path_listdir(dirpath: str, order_type: str = 'ASC', filetype: str = 'both') -> Union[SystemExit, list]:
    """排序指定路径中的文件/文件夹（仅第1层下级目录），并返回完整路径list
    :param dirpath: 文件夹路径
    :param order_type: 排序类型，'ASC' 升序或 'DESC' 降序
    :param filetype: 排序的文件类型，'file' 文件或 'folder' 文件夹或 'both' 两者
    :return: 排序后的完整路径list
    """
    listdir = os.listdir(dirpath)
    listdir_fullpath = [os.path.normpath(os.path.join(dirpath, i)) for i in listdir]
    listdir_fullpath_folder = [i for i in listdir_fullpath if os.path.isdir(i)]
    listdir_fullpath_file = [i for i in listdir_fullpath if os.path.isfile(i)]

    list_sorted_fullpath_folder = sort_list(listdir_fullpath_folder, order_type=order_type)
    list_sorted_fullpath_file = sort_list(listdir_fullpath_file, order_type=order_type)

    if filetype.lower() in ['both', 'file']:  # 如果排序类型是file也要返回整个list，但之后要删除folder项
        return list_sorted_fullpath_folder + list_sorted_fullpath_file
    elif filetype.lower() == 'folder':
        return list_sorted_fullpath_folder
    else:
        raise Exception(f'排序类型错误：{order_type}')


def _is_reversal_two_key(key_1: str, key_2: str):
    """排序两个键，决定是否在冒泡排序中掉转位置"""
    key_1_split = _split_key(key_1)
    key_2_split = _split_key(key_2)
    check_step = min(len(key_1_split), len(key_2_split))  # 检查步数，按最短的键的长度

    # 逐个对比两个键的每个字符，直到对比不同的字符，并返回对比结果
    for _ in range(check_step):
        split_1 = key_1_split.pop(0)
        split_2 = key_2_split.pop(0)
        if split_1 != split_2:
            splits = [split_1, split_2]
            if split_1.isdigit() and split_2.isdigit():  # 如果两个字符都是字符型数字，则按数值大小排序
                if int(split_1) == int(split_2):  # 处理特殊情况，如果两个字符转数值后相等，则需要按长度倒序排序，在Windows中00会排在0前
                    splits_sorted = sorted(splits, key=lambda x: len(x))[::-1]
                else:
                    splits_sorted = sorted(splits, key=lambda x: int(x))
            else:  # 否则按文本排序
                splits_sorted = sorted(splits, key=locale.strxfrm)
            if splits_sorted == splits:
                return False
            else:
                return True

    return False


def _split_key(key: str) -> list:
    """拆分key中的每一个字符（连续数字字符除外），返回拆分后元素组成的list"""
    pattern = r'(\d+)'  # 不需要拆分点. ，在排序时可以直接视其为字符
    split_pattern = re.split(pattern, key)

    key_split = []
    for part in split_pattern:
        if part.isdigit():
            key_split.append(part)
        else:
            key_split += list(part)

    return key_split



from PySide6.QtCore import *
from PySide6.QtGui import *
from PySide6.QtWidgets import *

from ._widget import WidgetLineOpenDeleteText


class ListWidgetFileList(QListWidget):
    """拖入文件/文件夹并显示在列表控件中，附带基础功能"""
    signal_update_list = Signal(list)

    def __init__(self, parent=None):
        super().__init__(parent)
        self.setAcceptDrops(True)
        self.setDragDropMode(QAbstractItemView.InternalMove)

        self.paths = []

    def _add_items(self, paths: list):
        """新增行项目"""
        for path in paths:
            if path not in self.paths:
                self.paths.append(path)

        self._refresh()

    def _refresh(self):
        """刷新项目"""
        self.clear()
        for path in self.paths:
            # 创建子控件
            item = QListWidgetItem()
            item_widget = WidgetLineOpenDeleteText()
            item_widget.set_text(path)
            item_widget.signal_delete.connect(self._delete_item)
            # 插入子控件
            end_index = self.count()
            self.insertItem(end_index + 1, item)
            self.setItemWidget(item, item_widget)

        # 发送更新后的list
        self.signal_update_list.emit(self.paths)

    def _delete_item(self, deleted_path):
        """删除行项目"""
        # 删除变量中的对应数据
        if deleted_path in self.paths:
            self.paths.remove(deleted_path)
        # 刷新
        self._refresh()

    def dragEnterEvent(self, event: QDragEnterEvent):
        if event.mimeData().hasUrls():
            event.acceptProposedAction()

    def dropEvent(self, event):
        if event.mimeData().hasUrls():
            urls = event.mimeData().urls()
            paths = [url.toLocalFile() for url in urls]
            self._add_items(paths)



import io
import zipfile
from typing import Union

import rarfile
from PIL import Image

from ..archive import get_infolist, read_image


def get_image_size(image_path: str):
    """获取图片的宽高
    :param image_path: str，本地图片路径"""
    image = Image.open(image_path)
    size = image.size
    return size


def get_image_filesize_from_archive(archive_path: str, image_path: str) -> int:
    """获取压缩文件中指定图片的文件大小（字节）
    :param archive_path: str，压缩文件路径
    :param image_path: str，压缩包内部图片路径
     :return: int，压缩文件中指定图片的文件大小（字节）"""
    infolist = get_infolist(archive_path)
    for info in infolist:
        info: Union[zipfile.ZipInfo, rarfile.RarInfo]
        path = info.filename
        if image_path.lower() == path.lower():
            size = info.file_size
            return size

    return 0  # 兜底


def get_image_size_from_archive(archive_path: str, image_path: str) -> tuple[int, int]:
    """获取压缩文件中指定图片的尺寸
    :param archive_path: str，压缩文件路径
    :param image_path: str，压缩包内部图片路径
     :return: tuple，(宽, 高）"""
    image_bytes = read_image(archive_path, image_path)
    image_stream = io.BytesIO(image_bytes)
    image_pil = Image.open(image_stream)
    size = image_pil.size
    return size



import inspect

from ._time import get_current_time


def print_function_info(mode: str = 'current'):
    """
    打印当前/上一个函数的信息
    :param mode: str，'current' 或 'last'
    """

    def _print_function_info(_stack_trace: inspect.FrameInfo):
        """打印函数信息"""
        # 打印当前时间
        print('当前时间:', get_current_time('%H:%M:%S'))

        # 获取函数名
        caller_function_name = _stack_trace.function
        print("调用函数名:", caller_function_name)

        # 获取文件路径
        caller_file_path = _stack_trace.filename
        print("调用文件路径:", caller_file_path)

    # return  # 不需要print信息时取消该备注

    # 获取当前帧对象
    # frame = inspect.currentframe()
    # 获取调用栈
    stack_trace = inspect.stack()  # stack_trace[0]为本函数，stack_trace[1]为调用本函数的函数
    if mode == 'current':  # 打印当前函数信息
        _print_function_info(stack_trace[1])
    elif mode == 'last':  # 打印上一个函数信息
        if len(stack_trace) >= 3:
            _print_function_info(stack_trace[2])


def get_subclasses(cls):
    """获取所有子类对象"""
    subclasses = []
    for subclass in cls.__subclasses__():
        subclasses.append(subclass)
        subclasses.extend(get_subclasses(subclass))
    return subclasses



from PySide6.QtCore import *
from PySide6.QtWidgets import *

from ._function import set_transparent_background


class WidgetLineOpenDeleteText(QWidget):
    """控件组合，打开按钮-删除按钮-文本框"""
    signal_delete = Signal(str)
    signal_open = Signal(str)

    def __init__(self, parent=None):
        super().__init__(parent)
        self.layout = QHBoxLayout(self)
        self.set_layout()
        self.toolButton_open = QToolButton()
        self._add_open_button()
        self.toolButton_delete = QToolButton()
        self._add_delete_button()
        self.text_label = QLabel()
        self._add_text_label()

        self.text = ''

    def set_text(self, text: str):
        """设置文本"""
        self.text_label.setText(text)
        self.text = text

    def set_tooltip(self, text: str):
        """设置文本提示"""
        self.text_label.setToolTip(text)

    def set_layout(self):
        self.layout.setSpacing(6)
        self.layout.setContentsMargins(0, 0, 0, 0)
        # self.layout.setStretch(1, 1)

    def _open(self):
        self.signal_open.emit(self.text)

    def _delete(self):
        self.deleteLater()
        self.signal_delete.emit(self.text)

    def _add_open_button(self):
        self.toolButton_open.setText('□')
        set_transparent_background(self.toolButton_open)
        self.layout.addWidget(self.toolButton_open)
        self.toolButton_open.clicked.connect(self._open)

    def _add_delete_button(self):
        self.toolButton_delete.setText('×')
        set_transparent_background(self.toolButton_delete)
        self.layout.addWidget(self.toolButton_delete)
        self.toolButton_delete.clicked.connect(self._delete)

    def _add_text_label(self):
        self.text_label.setText('')
        self.layout.addWidget(self.text_label)



import os
from typing import Union


def dedup_list(lst: list) -> list:
    """"剔除列表中的重复项"""
    list_dedup = []
    for i in lst:
        if i not in list_dedup:
            list_dedup.append(i)
    return list_dedup


def merge_intersection_item(items: Union[list, tuple, set]) -> list:
    """合并有交集的集合/列表/元组 [(1,2),{2,3},(5,6)]->[(1,2,3),(5,6)]
    :return: 示例 [(1,2),{2,3},(5,6)]->[(1,2,3),(5,6)]"""
    merged_list = []

    for i in range(len(items)):
        set_merged = False

        for j in range(len(merged_list)):
            if set(items[i]) & set(merged_list[j]):
                merged_list[j] = set(set(items[i]) | set(merged_list[j]))
                set_merged = True
                break

        if not set_merged:
            merged_list.append(items[i])

    return merged_list


def filter_child_folder(folder_list: list) -> list:
    """过滤文件夹列表中的所有子文件夹，返回剔除子文件夹后的list"""
    child_folder = set()
    for folder in folder_list:
        # 相互比对，检查是否为当前文件夹的下级
        for other_folder in folder_list:
            # 统一路径分隔符（os.path.normpath无法实现）
            other_folder_replace = os.path.normpath(other_folder).replace('/', '\\')
            folder_replace = os.path.normpath(folder).replace('/', '\\')
            compare_path = os.path.normpath(folder + os.sep).replace('/', '\\')
            if other_folder_replace.startswith(str(compare_path)) and other_folder_replace != folder_replace:
                child_folder.add(other_folder)

    for i in child_folder:
        if i in folder_list:
            folder_list.remove(i)

    return folder_list



import time
from typing import Union

import numpy
import pyautogui

# 默认参数
_DEFAULT_INTERVAL: float = 1  # 间隔时间
_DEFAULT_CONFIDENCE = 0.9  # 寻图精度
_DEFAULT_SCREENSHOT_IMAGE = 'screenshot.jpg'

"""
pyautogui库关于图像的操作调用了PyScreeze库
如果指定了confidence参数，则需要安装opencv库
"""


def screenshot_fullscreen(save_path: str = _DEFAULT_SCREENSHOT_IMAGE):
    """全屏截图并保存到本地
    :param save_path: str，保存的图片路径"""
    pyautogui.screenshot(save_path)

    return save_path


def screenshot_area(area: tuple, save_path: str = 'screenshot.png'):
    """指定区域截图并保存到本地图片
    :param area: tuple，截图区域(左上角X坐标值, 左上角Y坐标值, 右下角X坐标值, 右下角Y坐标值)
    :param save_path: str，保存的图片路径
    """
    region = (area[0], area[1], area[2] + area[0], area[3] + area[1])  # 转换area参数至pyautogui格式
    pyautogui.screenshot(save_path, region=region)

    return save_path


def _search_first_position(image_numpy: numpy.ndarray, confidence: float = _DEFAULT_CONFIDENCE) -> Union[tuple, None]:
    """在屏幕上搜索图片，获取第一次匹配到的中心点坐标
    :param image_numpy: numpy图片对象，需要搜索的图片
    :param confidence: float，搜索精度，0~1
    :return: 坐标轴元组(x, y)或None
    """
    position = pyautogui.locateCenterOnScreen(image_numpy, confidence=confidence)
    if position:
        loc = (position.x, position.y)
        return loc
    else:
        return None


def _search_all_position(image_numpy: numpy.ndarray, confidence: float = _DEFAULT_CONFIDENCE,
                         timeout: int = 60) -> list:
    """在屏幕上搜索图片，获取所有匹配到的中心点坐标
    :param image_numpy: numpy图片对象，需要搜索的图片
    :param confidence: float，搜索精度，0~1
    :param timeout: int，超时时间（秒）
    :return: list，包含坐标轴的元组[(x1, y1), (x2, y2)...]
    """
    time_start = time.time()
    positions = []
    while True:
        poss = pyautogui.locateAllOnScreen(image_numpy, confidence=confidence)
        for pos in poss:
            mid_x = pos.left + pos.width // 2
            mid_y = pos.top + pos.height // 2
            positions.append((mid_x, mid_y))
        if positions:
            break

        time_current = time.time()
        run_time = time_start - time_current
        if run_time >= timeout:
            break
        else:
            time.sleep(_DEFAULT_INTERVAL)

    return positions



import os
from typing import Union

import filetype
import win32com.client  # pywin32
from natsort import natsort

from lzytools.file._filepath import remove_subpaths


def get_size(path: str):
    """获取指定文件/文件夹的总大小（字节byte）
    :param path: str，文件/文件夹路径
    :return: int，总大小（字节byte）"""
    if not os.path.exists(path):
        raise Exception(f'路径不存在：{path}')

    if os.path.isdir(path):
        return get_dir_size(path)
    elif os.path.isfile(path):
        return os.path.getsize(path)
    else:
        return 0


def get_dir_size(dirpath: str) -> int:
    """获取指定文件夹的总大小（字节byte）
    :param dirpath: str，文件夹路径
    :return: int，总大小（字节byte）"""
    _folder_size = 0
    for _dirpath, dirnames, filenames in os.walk(dirpath):
        for item in filenames:
            filepath = os.path.join(_dirpath, item)
            _folder_size += os.path.getsize(filepath)

    return _folder_size


def get_files_in_dir(dirpath: str) -> list:
    """获取文件夹中所有文件的路径
    :param dirpath: str，文件夹路径"""
    files = []
    for _dirpath, dirnames, filenames in os.walk(dirpath):
        for filename in filenames:
            filepath_join = os.path.normpath(os.path.join(_dirpath, filename))
            files.append(filepath_join)

    return files


def get_files_in_paths(paths: list) -> list:
    """提取输入路径列表中所有文件路径"""
    # 删除路径中的子路径
    paths = remove_subpaths(paths)

    # 提取路径中的文件
    files = set()
    for path in paths:
        if os.path.isfile(path):
            files.add(path)
        elif os.path.isdir(path):
            child_files = get_files_in_dir(path)
            files.update(child_files)

    files = natsort.os_sorted(files)
    return files


def get_folders_in_dir(dirpath: str) -> list:
    """获取文件夹中所有文件夹的路径
    :param dirpath: str，文件夹路径"""
    folders = []
    for _dirpath, dirnames, filenames in os.walk(dirpath):
        for dirname in dirnames:
            filepath_join = os.path.normpath(os.path.join(_dirpath, dirname))
            folders.append(filepath_join)

    return folders


def guess_filetype(path) -> Union[str, None]:
    """判断文件类型
    :param path: str，文件路径"""
    if not os.path.isfile(path):
        return None

    kind = filetype.guess(path)
    if kind is None:
        return None

    type_ = kind.extension
    if type_:
        return type_
    else:
        return None


def get_first_multi_file_dirpath(dirpath: str) -> str:
    """找出文件夹中首个含多个下级文件/文件夹的文件夹路径（功能：解除套娃文件夹）
    :param dirpath: str，需要检查的文件夹路径
    :return: str，首个含多个下级文件/文件夹的文件夹路径
    """
    if not os.path.exists(dirpath):
        raise Exception("传入路径不存在")
    if not os.path.isdir(dirpath):
        raise Exception("传入路径不是文件夹")

    child_paths = os.listdir(dirpath)
    # 没有对空文件夹进行进一步检查
    if len(child_paths) == 1:  # 文件夹下级只有一个文件/文件夹
        child_path = os.path.normpath(os.path.join(dirpath, child_paths[0]))
        if os.path.isfile(child_path):  # 如果是文件，则直接返回结果
            return child_path
        else:  # 如果是文件夹，则递归
            return get_first_multi_file_dirpath(child_path)
    else:
        return dirpath


def get_shortcut_target_path(shortcut_path: str) -> str:
    """获取快捷方式指向的路径
    :param shortcut_path: str，快捷方式路径
    :return: 快捷方式指向的路径"""
    try:
        shell = win32com.client.Dispatch('WScript.Shell')
        shortcut = shell.CreateShortCut(shortcut_path)
        return shortcut.Targetpath
    except Exception as e:
        raise f'报错提示：{e}'



import ctypes
import os


def is_hidden_file(path: str):
    """路径对应的文件是否隐藏
    :param path: str，需要删除的路径"""
    get_file_attributes_w = ctypes.windll.kernel32.GetFileAttributesW
    file_attribute_hidden = 0x2
    invalid_file_attributes = -1

    def is_hidden(_file):
        # 获取文件属性
        attrs = get_file_attributes_w(_file)
        if attrs == invalid_file_attributes:
            # 文件不存在或无法访问
            return False

        return attrs & file_attribute_hidden == file_attribute_hidden

    return is_hidden(path)


def is_dup_filename(filename: str, check_dirpath: str) -> bool:
    """检查文件名在指定路径中是否已存在（检查重复文件名）
    :param filename: str，文件名（包含文件扩展名）
    :param check_dirpath: str，需要检查的文件夹路径
    :return: bool，是否在指定文件夹中存在重复文件名
    """
    filenames_in_dirpath = [i.lower() for i in os.listdir(check_dirpath)]
    return filename.lower() in filenames_in_dirpath



from ._check import *
from ._convert import *
from ._info import *
from ._read import *
from ._save import *
from ._similar import *

ImageFile.LOAD_TRUNCATED_IMAGES = True  # 允许加载截断的图像 防止报错OSError: image file is truncated



import random
import string

import unicodedata


def create_random_string(length: int = 16, lowercase: bool = True, uppercase: bool = True, digits: bool = True) -> str:
    """生成一个指定长度的随机字符串
    :param length: int，字符串长度
    :param lowercase: bool，小写英文字母
    :param uppercase: bool，大写英文字母
    :param digits: bool，数字
    :return: str，随机字符串"""
    characters = ''
    if lowercase:
        characters += string.ascii_lowercase
    if uppercase:
        characters += string.ascii_uppercase
    if digits:
        characters += string.digits
    if not characters:
        raise Exception('没有选择字符')

    random_string = ''.join(random.choices(characters, k=length))

    return random_string


def to_half_width_character(text: str):
    """将传入字符串转换为半角字符"""
    # 先将字符串进行Unicode规范化为NFKC格式（兼容性组合用序列）
    normalized_string = unicodedata.normalize('NFKC', text)

    # 对于ASCII范围内的全角字符，将其替换为对应的半角字符
    half_width_string = []
    for char in normalized_string:
        code_point = ord(char)
        if 0xFF01 <= code_point <= 0xFF5E:
            half_width_string.append(chr(code_point - 0xFEE0))
        else:
            half_width_string.append(char)

    return ''.join(half_width_string)


def to_full_width_character(text: str):
    """将传入字符串转换为全角字符"""
    # 将字符串进行Unicode规范化为NFKC格式（兼容性组合用序列）
    normalized_string = unicodedata.normalize('NFKC', text)

    # 对于ASCII范围内的字符，将其替换为对应的全角字符
    full_width_string = []
    for char in normalized_string:
        code_point = ord(char)
        if 0x0020 <= code_point <= 0x007E:
            full_width_string.append(chr(code_point + 0xFF00 - 0x0020))
        else:
            full_width_string.append(char)

    return ''.join(full_width_string)



import os
import shutil
import subprocess
import time

import send2trash

from ._filename import create_nodup_filename_standard_digital_suffix
from ._filepath import split_path
from ._info import get_first_multi_file_dirpath
from ..common import create_random_string


def delete(path: str, send_to_trash: bool = False) -> bool:
    """删除指定的文件/文件夹
    :param path: str，需要删除的路径
    :param send_to_trash: bool，是否删除至回收站
    :return: bool，是否成功删除"""
    if os.path.exists(path):
        try:
            if send_to_trash:
                send2trash.send2trash(path)
            else:
                if os.path.isfile(path):
                    os.remove(path)
                else:
                    shutil.rmtree(path)
            return True  # 成功执行删除操作的返回True
        except Exception as e:  # 报错PermissionError:文件被占用
            print(f'报错提示：{e}')
            return False
    return False


def delete_empty_folder(dirpath: str, send_to_trash: bool = True):
    """删除指定文件夹中的空文件夹（及其自身）
    :param dirpath: str，文件夹路径
    :param send_to_trash: bool，是否删除至回收站
    """
    _dirpaths = []

    # 提取所有文件夹路径
    for _dirpath, dirnames, filenames in os.walk(dirpath):
        _dirpaths.append(_dirpath)

    _dirpaths.insert(0, dirpath)  # 将其自身放于首位

    # 从后往前逐级删除
    for child_dirpath in _dirpaths[::-1]:
        if not os.listdir(child_dirpath):
            if send_to_trash:
                send2trash.send2trash(child_dirpath)
            else:
                os.rmdir(child_dirpath)


def set_hidden_attrib(path: str, is_hidden: bool = False) -> bool:
    """设置文件/文件及的隐藏属性
    :param path: str，文件/文件夹路径
    :param is_hidden: bool，是否隐藏文件/文件夹
    :return: bool，True为显示，False为隐藏"""
    if is_hidden:
        subprocess.run(['attrib', '+h', path])
        return False
    else:
        subprocess.run(['attrib', '-h', path])
        return True


def release_nesting_folder(check_path: str, target_dirpath: str) -> str:
    """解除套娃文件夹，将最深一级的非单层文件/文件夹移动至指定文件夹
    :param check_path: str，需要检查的路径
    :param target_dirpath: str，移动的目标文件夹
    :return: str，最终移动后的路径
    """
    if not os.path.exists(check_path):
        raise Exception('路径不存在')

    # 如果目标文件夹不存在，则新建该文件夹
    if not os.path.exists(target_dirpath):
        os.makedirs(target_dirpath)

    if not os.path.isdir(target_dirpath):
        raise Exception(f'传入文件夹参数错误，【{target_dirpath}】不是文件夹路径')

    # 提取需要移动的路径（如果传参是文件，则直接为该路径，如果传参是文件夹，则为最深一级非单层文件夹
    if os.path.isfile(check_path):
        need_move_path = check_path
    else:
        need_move_path = get_first_multi_file_dirpath(check_path)

    # 检查需要移动的路径是否和目标文件夹一致，如果一致，则不需要进行移动
    if need_move_path == target_dirpath:
        return need_move_path

    # 提取原始文件名，生成目标文件夹下无重复的文件夹名
    parent_dirpath, filetitle, file_extension = split_path(need_move_path)
    nodup_filename = create_nodup_filename_standard_digital_suffix(filetitle, target_dirpath, file_extension)

    # 移动前先重命名
    move_path_renamed = need_move_path
    _origin_filename = f'{filetitle}.{file_extension.strip(".")}'
    if nodup_filename == _origin_filename:  # 如果该文件名与原文件名一致，则不需要进行重命名
        pass
    else:  # 否则，先重命名为随机文件名（防止同目录存在重复文件名）
        _random_filename = f'{create_random_string()}.{create_random_string(4)}'
        _path_with_random_filename = os.path.normpath(os.path.join(parent_dirpath, _random_filename))
        move_path_renamed = _path_with_random_filename
        # 重命名时会遇到权限问题导致报错
        try:
            os.rename(need_move_path, _path_with_random_filename)
        except PermissionError:  # PermissionError: [WinError 5] 拒绝访问。尝试等待0.2秒后再次重命名
            time.sleep(0.2)
            try:
                os.rename(need_move_path, _path_with_random_filename)
            except Exception as e:
                raise e

    # 再进行移动
    try:
        shutil.move(move_path_renamed, target_dirpath)
    except OSError:  # OSError: [WinError 145] 目录不是空的。原始文件夹下有残留文件夹，如果为空则尝试直接删除
        delete_empty_folder(move_path_renamed)

    # 拼接最终路径
    final_path = os.path.normpath(os.path.join(target_dirpath, nodup_filename))
    delete_empty_folder(check_path)  # 如果原始文件夹为空，则直接删除

    return final_path



import random
import time

# 默认参数
_DEFAULT_WAIT_TIME = 1.0
_DEFAULT_WAIT_TIME_MIN = 1
_DEFAULT_WAIT_TIME_MAX = 2


def wait_time(_wait_time: float = _DEFAULT_WAIT_TIME):
    """等待指定时间
    :param _wait_time: float，等待时间"""
    if _wait_time == 0:
        _wait_time = 0.01

    time.sleep(_wait_time)

    return _wait_time


def wait_random_time(wait_time_min: int = _DEFAULT_WAIT_TIME_MIN, wait_time_max: int = _DEFAULT_WAIT_TIME_MAX):
    """等待随机时间
    :param wait_time_min: int，随机时间的下限
    :param wait_time_max: int，随机时间的上限"""
    wait_time_random = round(random.uniform(wait_time_min, wait_time_max), 2)

    if wait_time_random == 0:
        wait_time_random = 0.01

    time.sleep(wait_time_random)

    return wait_time_random



from PySide6.QtCore import *
from PySide6.QtWidgets import *


class SliderMoved(QSlider):
    """移动后发送新值信号"""
    signal_moved = Signal(int)

    def __init__(self, parent=None):
        super().__init__(parent)

    def moved(self, value: int):
        """发生移动事件"""
        # 固定滑动块为新值
        self.setValue(value)
        # 发送信号
        self._emit_signal(value)

    def _emit_signal(self, value: int):
        """发送信号"""
        self.signal_moved.emit(value)

    def mousePressEvent(self, event):
        super().mousePressEvent(event)
        value = QStyle.sliderValueFromPosition(self.minimum(), self.maximum(), event.x(), self.width())
        self.moved(value)



from PySide6.QtCore import *
from PySide6.QtGui import *
from PySide6.QtWidgets import *


class DialogScreenshot(QDialog):
    """添加全屏的半透明遮罩，实现截图操作操作"""
    signal_screenshot = Signal(bytes)

    def __init__(self):
        super().__init__()
        # 设置无边框
        self.setWindowFlags(self.windowFlags() | Qt.FramelessWindowHint)
        # 设置置顶
        self.setWindowFlags(Qt.WindowStaysOnTopHint | Qt.FramelessWindowHint)
        # 设置为全屏大小
        screen_geometry = QGuiApplication.primaryScreen().size()
        screen_width = screen_geometry.width()
        screen_height = screen_geometry.height()
        self.setGeometry(0, 0, screen_width, screen_height)
        # 设置半透明
        self.setWindowOpacity(0.5)  # 0~1，0为完全透明

        # 添加框选控件
        layout = QVBoxLayout(self)
        layout.setContentsMargins(0, 0, 0, 0)
        layout.setSpacing(0)
        widget = WidgetFrameSelect()
        widget.signal_screenshot_area.connect(self.screenshot)
        widget.rightClick.connect(self.close)
        layout.addWidget(widget)

    def screenshot(self, rect: QRect):
        # 去除遮罩
        self.setWindowOpacity(0)  # 0~1，0为完全透明
        # 截图
        screen = QApplication.primaryScreen()
        screenshot = screen.grabWindow(0, rect.x(), rect.y(), rect.width(), rect.height())
        screenshot_bytes = self.pixmap_to_bytes(screenshot)
        # 转为bytes图片对象，并发送信号
        self.signal_screenshot.emit(screenshot_bytes)

        self.close()

    @staticmethod
    def pixmap_to_bytes(pixmap: QPixmap) -> bytes:
        """将QPixmap转为bytes对象。"""
        byte_array = QByteArray()
        buffer = QBuffer(byte_array)
        buffer.open(QBuffer.WriteOnly)

        # 将 QPixmap 转为 PNG 格式保存到内存
        pixmap.save(buffer, "PNG")

        # 转换 QByteArray 为 bytes
        return byte_array.data()


class WidgetFrameSelect(QWidget):
    """在QWidget中实现截取区域的操作，发送截取区域的信号"""
    signal_screenshot_area = Signal(QRect)  # 发送截取区域QRect
    rightClick = Signal()  # 右键信号

    def __init__(self):
        super().__init__()
        # 初始化变量
        self.startPoint = None  # 截取起始点
        self.endPoint = None  # 截取终止点
        self.rubberBand = QRubberBand(QRubberBand.Rectangle, self)  # 橡皮筋控件（蚂蚁线）

    def mousePressEvent(self, event):
        if event.button() == Qt.LeftButton:  # 左键
            self.startPoint = event.pos()
            self.rubberBand.setGeometry(QRect(self.startPoint, QSize()))
            self.rubberBand.show()
        elif event.button() == Qt.RightButton:  # 右键
            self.rightClick.emit()

    def mouseMoveEvent(self, event):
        if self.startPoint:
            self.endPoint = event.pos()
            self.rubberBand.setGeometry(QRect(self.startPoint, self.endPoint).normalized())

    def mouseReleaseEvent(self, event):
        if event.button() == Qt.LeftButton and self.startPoint:
            self.endPoint = event.pos()
            self.rubberBand.hide()
            self.take_screenshot_area()  # 获取截取区域

    def take_screenshot_area(self):
        """获取截取区域
        注意：由于外部有半透明遮罩，所以不在该函数中直接进行截图操作"""
        rect = QRect(self.startPoint, self.endPoint).normalized()
        self.signal_screenshot_area.emit(rect)



import base64
from typing import Union

from PySide6.QtCore import *
from PySide6.QtGui import *
from PySide6.QtWidgets import *


def set_transparent_background(widget: QWidget):
    """设置Widget的背景为透明"""
    widget.setWindowFlags(Qt.FramelessWindowHint)  # 设置无边框
    widget.setAttribute(Qt.WA_TranslucentBackground)  # 设置透明背景
    widget.setStyleSheet("background-color: transparent; border: none;")


def calculate_keep_aspect_ratio_resize(qsize_widget: QSize, qsize_pic: QSize) -> QSize:
    """计算图片显示控件上时，为了保持图片纵横比而计算的控件的新尺寸"""
    label_width = qsize_widget.width()
    label_height = qsize_widget.height()
    pic_width = qsize_pic.width()
    pic_height = qsize_pic.height()

    label_rate = label_width / label_height
    pic_rate = pic_width / pic_height

    if label_rate >= pic_rate:  # 符合则按高缩放
        resize_height = label_height
        resize_width = int(pic_width / pic_height * resize_height)
        resize_qsize = QSize(resize_width, resize_height)
    else:  # 否则按宽缩放
        resize_width = label_width
        resize_height = int(pic_height / pic_width * resize_width)
        resize_qsize = QSize(resize_width, resize_height)

    """
    后续操作示例
    pixmap = pixmap.scaled(resize_qsize, spectRatioMode=Qt.KeepAspectRatio)  # 保持纵横比
    label.setPixmap(pixmap)
    """

    return resize_qsize


def base64_to_pixmap(image_base64: Union[bytes, str]) -> QPixmap:
    """base64图片转为pixmap对象
    :param image_base64: base64字节或字符串
    :return: QPixmap"""
    # 解码base64字节或字符串
    byte_data = base64.b64decode(image_base64)

    # 将字节数据转换为QPixmap
    pixmap = QPixmap()
    buffer = QByteArray(byte_data)
    byte_array_device = QBuffer(buffer)
    byte_array_device.open(QBuffer.ReadOnly)
    pixmap.loadFromData(byte_array_device.data())

    return pixmap


def bytes_to_pixmap(image_bytes: bytes) -> QPixmap:
    """bytes图片转为pixmap对象
    :param image_bytes: bytes图片对象
    :return: QPixmap"""
    image = QImage()
    image.loadFromData(image_bytes)
    pixmap = QPixmap.fromImage(image)

    return pixmap



from PySide6.QtCore import *
from PySide6.QtGui import *
from PySide6.QtWidgets import *


class ScrollAreaSmooth(QScrollArea):
    """平滑滚动的scrollArea"""

    def __init__(self, parent=None):
        super().__init__(parent)
        self.move_direction = 'v'  # 移动方向，v纵向h横向
        # 替换滚动条
        self.scrollbar_h = ScrollBarSmooth(self)
        self.scrollbar_v = ScrollBarSmooth(self)
        self.scrollbar_h.setOrientation(Qt.Horizontal)
        self.scrollbar_v.setOrientation(Qt.Vertical)
        self.setVerticalScrollBar(self.scrollbar_v)
        self.setHorizontalScrollBar(self.scrollbar_h)

    def set_move_direction(self, direction: str):
        """设置移动方向
        :param direction: str，v/纵向/h/横向"""
        if direction.lower() not in ['v', '纵向', 'h', '横向']:
            raise Exception(f'参数错误：{direction}')
        if direction.lower() in ['v', '纵向']:
            self.move_direction = 'v'
        elif direction.lower() in ['h', '横向']:
            self.move_direction = 'h'

    def wheelEvent(self, arg__1: QWheelEvent):
        if self.move_direction == 'v':
            self.scrollbar_v.scroll_value(-arg__1.angleDelta().y())
        elif self.move_direction == 'h':
            self.scrollbar_h.scroll_value(-arg__1.angleDelta().y())


class ScrollBarSmooth(QScrollBar):
    """实现平滑滚动的滚动条（在起点终点之间插值）"""
    MoveEvent = Signal(name='移动信号')
    MoveFinished = Signal(name='结束移动信号')
    AutoPlayStart = Signal(name='开始自动滚动')
    AutoPlayStop = Signal(name='停止自动滚动')

    def __init__(self, parent=None):
        super().__init__(parent=parent)
        # 设置缓出动画（用于滚轮等一般滚动）
        self.animal_smooth = QPropertyAnimation()
        self.animal_smooth.setTargetObject(self)
        self.animal_smooth.setPropertyName(b"value")
        self.animal_smooth.setEasingCurve(QEasingCurve.OutQuad)  # 二次缓出
        self.animal_smooth.setDuration(400)  # 动画时间 毫秒
        self.animal_smooth.finished.connect(self.MoveFinished.emit)

        # 设置线性动画（仅用于自动滚动）
        self.animal_linear = QPropertyAnimation()
        self.animal_linear.setTargetObject(self)
        self.animal_linear.setPropertyName(b"value")
        self.animal_linear.setEasingCurve(QEasingCurve.Linear)  # 二次缓出
        self.animal_linear.setDuration(400)  # 动画时间 毫秒

        # 默认动画为缓出动画
        self.animal = self.animal_smooth  # 差值动画，通过赋值不同的动画类型来改变平滑、线性状态

    def setValue(self, value: int):
        if value == self.value():
            return

        # 停止动画
        self.animal.stop()

        # 重新开始动画
        self.MoveEvent.emit()
        self.animal.setStartValue(self.value())
        self.animal.setEndValue(value)
        self.animal.start()

    def scroll_value(self, value: int):
        """滚动指定距离"""
        target_value = self.value() + value
        self.scroll_to_value(target_value)

    def scroll_to_value(self, value: int):
        """滚动到指定值"""
        value = min(self.maximum(), max(self.minimum(), value))  # 防止超限
        self.MoveEvent.emit()
        self.setValue(value)

    def set_type_smooth(self):
        """设置平滑滚动"""
        self.animal.stop()
        self.animal = self.animal_smooth
        self.AutoPlayStop.emit()

    def set_type_linear(self, value: int, duration: float):
        """设置线性滚动
        :param value: int，需要滚动至的位置
        :param duration: float，滚动时间，秒"""
        self.animal.stop()
        self.animal_linear.setDuration(int(duration * 1000))
        self.animal = self.animal_linear
        self.scroll_to_value(value)
        self.AutoPlayStart.emit()

    def _set_smooth_animal_duration(self, duration: int):
        """设置缓出动画的动画时间
        :param duration: int，毫秒"""
        self.animal_smooth.stop()
        self.animal_smooth.setDuration(duration)  # 动画时间 毫秒

    def _set_linear_animal_duration(self, duration: int):
        """设置线性动画的动画时间
        :param duration: int，毫秒"""
        self.animal_linear.stop()
        self.animal_linear.setDuration(duration)  # 动画时间 毫秒

    def is_autoplay_running(self) -> bool:
        """是否正在进行自动播放"""
        # 通过判断滑动动画来判断是否正在进行自动播放
        if self.animal.easingCurve().type() == QEasingCurve.Linear:
            return True
        else:
            return False

    def mousePressEvent(self, event):
        self.set_type_smooth()
        super().mousePressEvent(event)

    def mouseReleaseEvent(self, event):
        self.set_type_smooth()
        super().mouseReleaseEvent(event)

    def mouseMoveEvent(self, event):
        self.set_type_smooth()
        super().mouseMoveEvent(event)

    def wheelEvent(self, event):
        self.set_type_smooth()
        self.scroll_value(-event.angleDelta().y())



from typing import Union

import imagehash
import numpy
from PIL import ImageFile


def calc_hash(image: ImageFile, hash_size: int = 8) -> dict:
    """计算图片的3种图片Hash值
    :param image: PIL.ImageFile图片对象
    :param hash_size: 计算的图片Hash值的边长
    :return: dict，{'ahash':None,'phash':None,'dhash':None}
    """
    hash_dict = {'ahash': None, 'phash': None, 'dhash': None}
    # 计算均值哈希
    ahash = imagehash.average_hash(image, hash_size=hash_size)
    ahash_str = numpy_hash_to_str(ahash)
    hash_dict['ahash'] = ahash_str
    # 感知哈希
    phash = imagehash.phash(image, hash_size=hash_size)
    phash_str = numpy_hash_to_str(phash)
    hash_dict['phash'] = phash_str
    # 差异哈希
    dhash = imagehash.dhash(image, hash_size=hash_size)
    dhash_str = numpy_hash_to_str(dhash)
    hash_dict['dhash'] = dhash_str

    return hash_dict


def numpy_hash_to_str(numpy_hash: Union[imagehash.NDArray, imagehash.ImageHash]):
    """将numpy数组形式的图片Hash值(imagehash.hash)转换为01组成的字符串
    :param numpy_hash: numpy数组形式的图片Hash值
    :return: str，01组成的字符串"""
    if not numpy_hash:
        return None
    if isinstance(numpy_hash, imagehash.ImageHash):
        numpy_hash = numpy_hash.hash

    hash_str = ''
    for row in numpy_hash:
        for col in row:
            if col:
                hash_str += '1'
            else:
                hash_str += '0'

    return hash_str


def calc_hash_hamming_distance(hash_1: str, hash_2: str):
    """计算两个01字符串形式的图片Hash值的汉明距离"""
    hamming_distance = sum(ch1 != ch2 for ch1, ch2 in zip(hash_1, hash_2))
    return hamming_distance


def calc_hash_similar(hash_1: str, hash_2: str):
    """计算两个01字符串形式的图片Hash值的相似度（0~1)"""
    hash_int1 = int(hash_1, 2)
    hash_int2 = int(hash_2, 2)
    # 使用异或操作计算差异位数
    diff_bits = bin(hash_int1 ^ hash_int2).count('1')
    # 计算相似性
    similarity = 1 - diff_bits / len(hash_1)

    return similarity


def calc_ssim(image_numpy_1: numpy.ndarray, image_numpy_2: numpy.ndarray) -> float:
    """计算两张图片的SSIM相似值（0~1，越大越相似）
    :param image_numpy_1: numpy图片对象
    :param image_numpy_2: numpy图片对象
    :return: float，SSIM相似值（0~1，越大越相似）"""
    # 计算均值、方差和协方差
    mean1, mean2 = numpy.mean(image_numpy_1), numpy.mean(image_numpy_2)
    var1, var2 = numpy.var(image_numpy_1), numpy.var(image_numpy_2)
    covar = numpy.cov(image_numpy_1.flatten(), image_numpy_2.flatten())[0][1]

    # 设置常数
    c1 = (0.01 * 255) ** 2
    c2 = (0.03 * 255) ** 2

    # 计算SSIM
    numerator = (2 * mean1 * mean2 + c1) * (2 * covar + c2)
    denominator = (mean1 ** 2 + mean2 ** 2 + c1) * (var1 + var2 + c2)
    ssim = numerator / denominator

    return ssim



import re
import subprocess
from typing import Tuple

from lzytools.common import send_data_to_socket

"""
| Code | Meaning                         | 翻译                                               |
|------|---------------------------------|--------------------------------------------------|
| 0    | No error                        | 无错误                                              |
| 1    | Warning (Non fatal error(s)).   | 警告 (非致命错误)。<br/>例如，一个或多个文件被其他某个应用程序锁定，因此它们没有被压缩。 |
| 2    | Fatal error                     | 致命错误                                             |
| 7    | Command line error              | 命令行错误                                            |
| 8    | Not enough memory for operation | 内存不足，无法进行操作                                      |
| 255  | User stopped the process        | 用户已停止该进程                                         |
"""

_PATH_7ZIP = None
_HOST = '127.0.0.1'
_PORT = '7219'


def check_7zip_path():
    """检查设置的7zip路径"""
    if not _PATH_7ZIP:
        raise Exception('未指定7Zip路径')


def subprocess_7zip_lt(command_type: str, filepath: str, password: str,
                       inside_path=None) -> subprocess.CompletedProcess:
    """外部调用7zip（l或t指令），返回结果
    :param command_type: l或t指令
    :param filepath: 测试的压缩文件路径
    :param password: 测试的解压密码
    :param inside_path: 指定测试的内部文件路径（只在t指令时使用）
    :return: 7zip调用结果"""
    check_7zip_path()

    command = [_PATH_7ZIP,
               command_type,
               filepath,
               "-p" + password]
    if command_type == 't' and inside_path:  # 在使用t指令时，指定测试的内部路径可以加快测速速度
        command.append(inside_path)
    print(f'调用7zip：{" ".join(command)}')
    process = subprocess.run(command,
                             stdout=subprocess.PIPE,
                             stderr=subprocess.PIPE,
                             creationflags=subprocess.CREATE_NO_WINDOW,
                             text=True,
                             universal_newlines=True)

    return process


def analyse_return(process: subprocess.CompletedProcess) -> Tuple[int, str]:
    """分析调用7zip的返回结果
    :param process: 7zip调用结果
    :return: 7zip返回码，返回码对应解释"""
    return_code = process.returncode
    if return_code == 0:
        return 0, 'No error'
    elif return_code == 1:
        return 1, 'Warning (Non fatal error)'
    elif return_code == 2:
        output = str(process.stderr) + str(process.stdout)
        if 'Wrong password' in output:
            return 2, 'Wrong password'
        elif 'Missing volume' in output:
            return 2, 'Missing volume'
        elif 'Cannot open the file as' in output:
            return 2, 'Cannot open the file'
        else:
            return 2, 'Unknown error'
    elif return_code == 7:
        return 7, 'Command line error'
    elif return_code == 8:
        return 8, 'Not enough memory for operation'
    elif return_code == 255:
        return 255, 'User stopped the process'
    else:
        return 2, 'Unknown error'  # 兜底


def read_stdout(process: subprocess.CompletedProcess):
    """读取返回结果中的stdout信息，提取文件列表"""
    stdout = process.stdout
    files_dict = {'filetype': None, 'paths': None}
    if stdout:
        text_splits = stdout.splitlines()
    else:
        return files_dict

    # 提取文件类型
    cut_splits = [i for i in text_splits if i.startswith('Type = ')]
    if cut_splits:
        cut_text = [i for i in text_splits if i.startswith('Type = ')][0]
        filetype = cut_text[len('Type = '):]
        files_dict['filetype'] = filetype

    # 提取内部文件路径
    start_index = None
    end_index = None
    for index, i in enumerate(text_splits):
        if i.startswith('   Date'):
            start_index = index
        if i.startswith('----------'):
            end_index = index
    if start_index or end_index:
        column_name_index = text_splits[start_index].find('Name')
        cut_text = text_splits[start_index + 2:end_index]
        paths = [i[column_name_index:] for i in cut_text if 'D....' not in i]
        files_dict['paths'] = paths

    return files_dict


def subprocess_7zip_x(filepath: str, password: str, extract_dirpath: str, filter_rule: list) -> tuple[int, str]:
    """外部调用7zip（x指令），返回结果
    :param filepath: 压缩文件路径
    :param password: 解压密码
    :param extract_dirpath: 解压路径
    :param filter_rule: 过滤器，不解压的文件规则
    :return: 7zip返回码，返回码对应解释"""
    # 同时读取stdout和stderr会导致管道堵塞，需要将两个输出流重定向至同一个管道中，使用switch：'bso1','bsp1',bse1'
    command = ([_PATH_7ZIP, 'x', '-y', filepath,
                '-bsp1', '-bse1', '-bso1',
                '-o' + extract_dirpath,
                '-p' + password]
               + filter_rule)

    print(f'调用7zip：{" ".join(command)}')
    process = subprocess.Popen(command,
                               stdout=subprocess.PIPE,
                               stderr=subprocess.PIPE,
                               creationflags=subprocess.CREATE_NO_WINDOW,
                               text=True,
                               universal_newlines=True)

    # 读取输出流
    # （使用subprocess.Popen调用7zip时，返回码为2时的报错信息为"<_io.TextIOWrapper name=4 encoding='cp936'>"，
    # 无法正确判断错误事件，所以需要在实时输出的输出流中进行读取判断）
    code_text = None  # 返回码对应文本
    pre_progress = 0  # 解压进度
    is_read_stderr = True  # 是否读取stderr流，出现报错事件/读取到进度信息后不再需要读取
    is_read_progress = True  # 是否读取进度信息，出现报错事件后不再需要读取
    while True:
        try:
            output = process.stdout.readline()
            print(f'7zip实时输出文本：\n{output}')  # 测试用
        except UnicodeDecodeError:  # UnicodeDecodeError: 'gbk' codec can't decode byte 0xaa in position 32: illegal multibyte sequence
            output = ''
        if output == '' and process.poll() is not None:  # 读取到空文本或返回码时，结束读取操作
            break

        # 读取错误事件
        if is_read_stderr and output:
            is_wrong_password = re.search('Wrong password', output)
            is_missing_volume = re.search('Missing volume', output)
            is_cannot_open_the_file = re.search('Cannot open the file as', output)
            if is_wrong_password:
                code_text = 'Wrong password'
                is_read_stderr = False
                is_read_progress = False
            elif is_missing_volume:
                code_text = 'Missing volume'
                is_read_stderr = False
                is_read_progress = False
            elif is_cannot_open_the_file:  # 如果是为指定后缀的问题，7zip会自动尝试以正确格式进行解压，不需要停止读取
                code_text = 'Cannot open the file as'

        # 读取进度事件
        if is_read_progress and output:
            # 单文件进度输出示例：34% - 061-090；多文件进度输出示例：19% 10 - 031-060。适用正则表达式 '(\d{1,3})% *\d* - '
            # 部分压缩文件的输出示例：80% 13。适用正则表达式 '(\d{1,3})% *\d*'
            match_progress = re.search(r'(\d{1,3})% *\d*', output)
            if match_progress:
                is_read_stderr = False
                current_progress = int(match_progress.group(1))  # 提取进度百分比（不含%）
                if current_progress > pre_progress:
                    send_data_to_socket(current_progress, _HOST, _PORT)  # 使用socket传递数据
                    pre_progress = current_progress  # 更新进度

    # 结束后读取返回码
    return_code = process.poll()
    if return_code == 0:
        return 0, 'No error'
    elif return_code == 1:
        return 1, 'Warning (Non fatal error)'
    elif return_code == 2:
        if not code_text:
            code_text = 'Unknown error'  # 兜底
        return 2, code_text
    elif return_code == 7:
        return 7, 'Command line error'
    elif return_code == 8:
        return 8, 'Not enough memory for operation'
    elif return_code == 255:
        return 255, 'User stopped the process'
    else:  # 兜底
        return 2, 'Unknown error'  # 兜底



from PySide6.QtCore import *


class QtimerSingleShot(QTimer):
    """单例单次触发的计时器"""
    _instance = None
    _is_init = False
    timeStart = Signal(name='开始信号')

    def __new__(cls, *args, **kwargs):
        if cls._instance is None:
            cls._instance = super().__new__(cls)
        return cls._instance

    def __init__(self, parent=None):
        if not self._is_init:
            super().__init__(parent)
            self._is_init = True
        self.setSingleShot(True)  # 设置为单次触发

    def start(self):
        super().start()
        self.timeStart.emit()



import os
from typing import Union

import cv2
import numpy


def read_image_to_numpy(image_path: str) -> numpy.ndarray:
    """读取本地图片，返回numpy图片对象
    :param image_path: 图片路径
    :return: numpy图片对象"""
    image_numpy = cv2.imdecode(numpy.fromfile(image_path, dtype=numpy.uint8), -1)

    return image_numpy


def read_image_to_bytes(image_path: str) -> bytes:
    """读取本地图片，返回bytes图片对象
    :param image_path: 图片路径
    :return: bytes图片对象"""
    if os.path.exists(image_path):
        with open(image_path, 'rb') as file:
            image_bytes = file.read()
    else:
        image_bytes = rb''

    return image_bytes


def read_image(image_path: str, type_: str = 'bytes') -> Union[bytes, numpy.ndarray]:
    """读取本地图片，返回bytes图片对象
    :param image_path: 图片路径
    :param type_: 返回的数据类型，'bytes'/'numpy'
    :return: bytes/numpy图片对象"""
    _TYPE = ['bytes', 'numpy']
    if type_.lower() not in _TYPE:
        raise Exception('读取类型输入错误')

    if type_.lower() == 'bytes':
        img = read_image_to_bytes(image_path)
    else:
        img = read_image_to_numpy(image_path)

    return img



import pickle
import socket

from PySide6.QtCore import *


class ThreadListenSocket(QThread):
    """监听本地端口的子线程"""
    signal_receive_args = Signal(list)

    def __init__(self, parent=None):
        super().__init__(parent)
        self.host = '127.0.0.1'  # 主机地址
        self.port = '9527'  # 端口
        self.client_limit_count = 2  # 同时连接的客户端上限

    def set_host(self, host: str):
        """设置主机地址"""
        self.host = host

    def set_port(self, port: str):
        """设置端口"""
        self.port = port

    def set_client_limit_count(self, count: int):
        """设置同时连接的客户端上限"""
        self.client_limit_count = count

    def get_host(self):
        """获取绑定的主机地址"""
        return self.host

    def get_port(self):
        """获取绑定的端口"""
        return self.port

    def get_client_limit_count(self):
        """获取同时连接的客户端上限"""
        return self.client_limit_count

    def run(self):
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sock.bind((self.host, self.port))
        sock.listen(self.client_limit_count)
        while True:
            connection, client_address = sock.accept()
            try:
                # 接收数据
                data = connection.recv(1024)
                if data:
                    args = pickle.loads(data)
                    # 打印接收到的参数
                    print(f'接收参数：{args}')
                    self.signal_receive_args.emit(args)
            finally:
                connection.close()  # 关闭连接



import zipfile
from typing import Union

import rarfile


def read_archive(archive_path: str) -> Union[zipfile.ZipFile, rarfile.RarFile, bool]:
    """读取压缩文件，并返回压缩文件对象（仅支持zip和rar）
    :param archive_path: str，压缩文件路径
    :return: 返回zip/rar对象，或正确读取文件时返回False"""
    try:
        archive = zipfile.ZipFile(archive_path)
    except zipfile.BadZipFile:
        try:
            archive = rarfile.RarFile(archive_path)
        except rarfile.NotRarFile:
            return False
    except FileNotFoundError:
        return False

    return archive


def get_infolist(archive_path: str) -> list[zipfile.ZipInfo]:
    """获取压缩文件的内部信息list（仅支持zip和rar）
    :param archive_path: str，压缩文件路径
    :return: list，zipfile/rarfile库读取的压缩文件的infolist"""
    archive = read_archive(archive_path)
    if not archive:
        raise Exception('未正确读取文件，该文件不是压缩文件或文件不存在')

    infolist = archive.infolist()  # 中文等字符会变为乱码

    archive.close()

    return infolist


def get_structure(archive_path: str) -> list:
    """获取压缩文件的内部文件结构（仅支持zip和rar）
    :param archive_path: str，压缩文件路径
    :return: list，内部文件和文件夹，按层级排序"""
    infolist = get_infolist(archive_path)
    filenames = [i.filename for i in infolist]

    return filenames


def get_real_size(archive_path: str) -> int:
    """获取一个压缩文件的内部文件大小（解压后的原始文件大小）
    :param archive_path: str，压缩文件路径
    :return: int，压缩包内部文件的实际大小（字节）"""
    total_size = 0
    infolist = get_infolist(archive_path)
    for info in infolist:
        info: Union[zipfile.ZipInfo, rarfile.RarInfo]
        total_size += info.file_size

    return total_size


def read_image(archive_path: str, image_path: str) -> bytes:
    """读取压缩文件中的指定图片，返回一个bytes图片对象
    :param archive_path: str，压缩文件路径
    :param image_path: str，压缩包内部图片路径"""
    archive = read_archive(archive_path)
    if not archive:
        raise Exception('未正确读取文件，该文件不是压缩文件或文件不存在')

    try:
        img_data = archive.read(image_path)
    except KeyError:
        raise Exception('压缩文件中不存在该文件')

    archive.close()

    return img_data



import re
from typing import Union

from lzytools.file._check import is_dup_filename

# WINDOWS系统文件命名规则：文件和文件夹不能命名为“.”或“..”，也不能包含以下任何字符: \ / : * ? " < > |
_ILLEGAL_CHARACTERS = ['\\', '/', ':', '*', '?', '"', '<', '>', '|']


def is_legal_filename(filename: str) -> bool:
    """检查文件名是否符合Windows的文件命名规范
    :param filename: str，仅文件名（不含文件扩展名和文件父路径）
    """
    # 检查.（强制文件名不能以.开头）
    if filename[0] == '.':
        return False

    # 检查非法字符
    for key in _ILLEGAL_CHARACTERS:
        if key in filename:
            return False

    return True


def replace_illegal_filename(filename: str, replace_str: str = '') -> Union[str, bool]:
    """替换文件名中的非法字符串
    :param filename: str，仅文件名（不含文件扩展名和文件父路径）
    :param replace_str: str，用于替换的新字符
    """
    # 检查用于替换的字符是否属于非法字符
    if replace_str in _ILLEGAL_CHARACTERS:
        raise Exception(f'替换字符非法：{replace_str}')

    # 替换非法字符
    for word in _ILLEGAL_CHARACTERS:
        filename = filename.replace(word, replace_str)

    # 替换.（强制文件名不能以.开头）
    while filename[0] == '.':
        filename = filename[1:]

    filename = filename.strip()
    if not filename:
        raise Exception(f'结果文件名非法：{filename}')

    return filename


def remove_suffix(filetitle: str, suffix: str = None) -> str:
    """提取无后缀的文件名（剔除（1）等后缀和自定义后缀）
    :param filetitle: str，文件名（不包含文件扩展名）
    :param suffix: str，自定义的后缀（若存在）
    :return: str，提取出的无后缀的文件名（不包含文件扩展名）"""
    # 剔除(1)等后缀
    filetitle = re.sub(r'\s*\(\d+\)\s*$', '', filetitle)
    # 剔除（1）等后缀
    filetitle = re.sub(r'\s*（\d+）\s*$', '', filetitle)
    # 剔除自定义后缀+数字的组合
    if suffix:
        filetitle = re.sub(rf'\s*{suffix}\s*\d+\s*$', '', filetitle)

    return filetitle.strip()


def create_nodup_filename_standard_digital_suffix(filetitle: str, check_dirpath: str,
                                                  filename_extension: str = None) -> str:
    """生成文件名在目标文件夹中非重复的文件名（统一数字后缀的文件名，(1)（1）等后缀）
    :param filetitle: str，文件名（不包含文件扩展名）
    :param filename_extension: str，文件扩展名（如果检查的文件名是文件的文件名，则必须使用该参数）
    :param check_dirpath: str，目标文件夹路径
    :return: str，非重复的文件名（包含文件扩展名）"""
    # 剔除后缀
    filetitle = remove_suffix(filetitle)

    # 标准化文件扩展名
    if filename_extension:
        filename_extension = filename_extension.strip().strip('.')

    # 组合文件名
    if filename_extension:
        filename = f'{filetitle}.{filename_extension}'  # 假设为文件的文件名
    else:
        filename = filetitle  # 假设为文件夹的文件名

    # 检查重复文件名
    if is_dup_filename(filename, check_dirpath):
        # 生成无重复的文件名，按照Windows重复文件名规则，一直循环后缀编号累加，直到不存在重复文件名
        count = 1
        while True:
            # 组合文件名
            if filename_extension:
                filename = f'{filetitle} ({count}).{filename_extension}'  # 假设为文件的文件名
            else:
                filename = f'{filetitle} ({count})'  # 假设为文件夹的文件名

            # 检查
            if is_dup_filename(filename, check_dirpath):
                count += 1
            else:
                break

    return filename


def create_nodup_filename_custom_suffix(filetitle: str, check_dirpath: str, add_suffix: str,
                                        filename_extension: str = None) -> str:
    """生成指定文件名在目标文件夹中非重复的文件名（可指定目标文件名）
    :param filetitle: str，文件名（不包含文件扩展名）
    :param filename_extension: str，文件扩展名（如果检查的文件名是文件的文件名，则必须使用该参数）
    :param check_dirpath: str，目标文件夹路径
    :param add_suffix: 存在重复文件名时在文件名后添加的后缀
    :return: str，非重复的文件名（包含文件扩展名）"""
    # 剔除原始后缀
    filetitle = remove_suffix(filetitle, add_suffix)

    # 标准化文件扩展名
    if filename_extension:
        filename_extension = filename_extension.strip().strip('.')

    # 组合文件名
    if filename_extension:
        filename = f'{filetitle}.{filename_extension}'  # 假设为文件的文件名
    else:
        filename = filetitle  # 假设为文件夹的文件名

    # 检查重复文件名
    if is_dup_filename(filename, check_dirpath):
        # 生成无重复的文件名，一直循环后缀编号累加，直到不存在重复文件名
        count = 1
        while True:
            # 组合文件名
            if filename_extension:
                filename = f'{filetitle}{add_suffix}{count}.{filename_extension}'  # 假设为文件的文件名
            else:
                filename = f'{filetitle}{add_suffix}{count}'  # 假设为文件夹的文件名

            # 检查
            if is_dup_filename(filename, check_dirpath):
                count += 1
            else:
                break

    return filename



import time

import pyautogui
import pyperclip

# 默认参数
_DEFAULT_INTERVAL: float = 0.1  # 间隔时间
_DEFAULT_PRESSES: int = 1  # 重复次数


def press_text(message: str, presses: int = _DEFAULT_PRESSES, interval: float = _DEFAULT_INTERVAL):
    """输入字符串（通过复制粘贴实现）
    :param message: str，输入的文本
    :param presses: int，重复次数
    :param interval: float，两次输入之间的间隔时间
    """
    for _ in range(presses):
        pyperclip.copy(message)
        pyautogui.hotkey('ctrl', 'v')
        time.sleep(interval)

    return message


def press_keys(keys: list, presses: int = _DEFAULT_PRESSES, interval: float = _DEFAULT_INTERVAL):
    """敲击指定键盘按键
    :param keys: list，单个或多个键盘按键，需要指定的按键名称
    :param presses: int，重复次数
    :param interval: float，两次输入之间的间隔时间
    """
    pyautogui.press(keys=keys, presses=presses, interval=interval)

    return keys


def press_down(key: str):
    """按下键盘按键
    :param key: str，单个键盘按键，需要指定的按键名称"""
    pyautogui.keyDown(key)

    return key


def release(key: str):
    """释放键盘按键
    :param key: str，单个键盘按键，需要指定的按键名称"""
    pyautogui.keyUp(key)

    return key


def press_hotkey(hotkeys: list):
    """按下组合键盘按键，实现热键操作
    :param hotkeys: list，单个或多个键盘按键，需要指定的按键名称"""
    pyautogui.hotkey(hotkeys)

    return hotkeys



import os
from typing import Union

import filetype
from PySide6.QtCore import *
from PySide6.QtGui import *
from PySide6.QtWidgets import *

from ._function import calculate_keep_aspect_ratio_resize


class LineEditDropFiles(QLineEdit):
    """支持拖入多个文件/文件夹的文本框，设置文本为拖入的路径，并发送信号传递拖入路径的list"""

    signal_path_dropped = Signal(list)

    def __init__(self, parent=None):
        super().__init__(parent)
        self.setAcceptDrops(True)
        self.setReadOnly(True)
        self.setPlaceholderText('拖入文件到此处...')

        self.path_dropped = []

    def update_paths(self, paths: list):
        """更新路径
        :param paths: list，包含完整路径的列表"""
        # 列表去重
        paths = list(dict.fromkeys(paths))
        # 重置参数
        self.path_dropped = paths
        # 更新文本
        self._update_text(paths)
        # 发送信号
        self._emit_signal(paths)

    def _update_text(self, paths: Union[list, str]):
        """更新文本"""
        if isinstance(paths, str):
            paths = [paths]
        self.setText(';'.join(paths))
        self.setToolTip('/n'.join(paths))

    def _emit_signal(self, paths: Union[list, str]):
        """发送信号"""
        if isinstance(paths, str):
            paths = [paths]
        self.signal_path_dropped.emit(paths)

    def dragEnterEvent(self, event: QDragEnterEvent):
        if event.mimeData().hasUrls():
            event.acceptProposedAction()

    def dropEvent(self, event: QDropEvent):
        urls = event.mimeData().urls()
        if urls:
            paths = []
            for index in range(len(urls)):
                path = urls[index].toLocalFile()
                paths.append(path)
            self.update_paths(paths)


class LineEditDropFile(LineEditDropFiles):
    """支持拖入单个文件/文件夹的文本框，设置文本为拖入的路径，并发送信号传递拖入路径的list
    新增功能：定时检查路径有效性"""

    def __init__(self, parent=None):
        super().__init__(parent)
        # 设置一个QTime，定时检查路径有效性
        self.is_exists = True
        self.stylesheet_not_exists = 'border: 1px solid red;'
        self.qtimer_check_path_exists = QTimer()
        self.qtimer_check_path_exists.timeout.connect(self._check_path_exists)
        self.qtimer_check_path_exists.setInterval(5000)  # 默认定时5秒
        self.qtimer_check_path_exists.start()

    def update_paths(self, paths: list):
        """更新路径
        :param paths: list，包含完整路径的列表"""
        # 提取首个路径
        path = paths[0]
        # 重置参数
        self.path_dropped = path
        # 更新文本
        self._update_text(path)
        # 发送信号
        self._emit_signal(path)

    def set_check_interval(self, second: float):
        """设置定时检查路径有效性的时间间隔"""
        self.qtimer_check_path_exists.setInterval(int(second * 1000))

    def set_stylesheet_not_exists(self, stylesheet: str):
        """设置路径不存在时的文本框样式"""
        self.stylesheet_not_exists = stylesheet

    def _check_path_exists(self):
        """检查路径有效性"""
        if self.path_dropped:
            if os.path.exists(self.path_dropped):
                self.is_exists = True
                self.setStyleSheet('')
            else:
                self.is_exists = False
                self.setStyleSheet(self.stylesheet_not_exists)
        else:
            self.is_exists = False


class LabelDropFiles(QLabel):
    """支持拖入多个文件/文件夹的标签控件，并发送信号传递拖入路径的list"""

    signal_path_dropped = Signal(list)

    def __init__(self, parent=None):
        super().__init__(parent)
        self.setAcceptDrops(True)
        self.setScaledContents(True)

        self.path_dropped = []

    def update_paths(self, paths: list):
        """更新路径
        :param paths: list，包含完整路径的列表"""
        # 列表去重
        paths = list(dict.fromkeys(paths))
        # 重置参数
        self.path_dropped = paths
        # 发送信号
        self._emit_signal(paths)

    def _emit_signal(self, paths: Union[list, str]):
        """发送信号"""
        if isinstance(paths, str):
            paths = [paths]
        self.signal_path_dropped.emit(paths)

    def dragEnterEvent(self, event):
        if event.mimeData().hasUrls():
            event.accept()

    def dropEvent(self, event):
        if event.mimeData().hasUrls():
            urls = event.mimeData().urls()
            paths = [url.toLocalFile() for url in urls]
            self.update_paths(paths)


class LabelDropFilesTip(LabelDropFiles):
    """支持拖入多个文件/文件夹的标签控件，并发送信号传递拖入路径的list
    新增功能：拖入时会提示"""

    def __init__(self, parent=None):
        super().__init__(parent)
        self.icon_drop = ''  # 拖入时的图标
        self.icon_last = None  # 拖入前的图标

    def set_drop_icon(self, icon_path: str):
        """拖入图标路径"""
        self.icon_drop = ''
        if os.path.exists(icon_path):
            self.icon_drop = icon_path

    def dragEnterEvent(self, event):
        if event.mimeData().hasUrls():
            event.accept()
            if self.icon_drop:
                self.icon_last = self.pixmap()
                self.setPixmap(QPixmap(self.icon_drop))  # 拖入时修改图标
        else:
            event.ignore()

    def dragLeaveEvent(self, event):
        if self.icon_drop:
            self.setPixmap(QPixmap(self.icon_last))  # 完成拖入后变回原图标
            self.icon_last = None


class LabelDropImageShow(LabelDropFiles):
    """拖入单个图片文件并显示"""

    def __init__(self, parent=None):
        super().__init__(parent)

    def update_paths(self, paths: list):
        """更新路径
        :param paths: list，包含完整路径的列表"""
        # 提取首个路径并进行检查
        path = paths[0]
        self.path_dropped = ''
        if os.path.isfile(path) and filetype.is_image(path):
            self.path_dropped = path

        # 显示图片
        if self.path_dropped:
            self._show_image(self.path_dropped)
        else:
            self._clear_image()

        # 发送信号
        self._emit_signal(paths)

    def _show_image(self, image_path: str):
        pixmap = QPixmap(image_path)
        resize = calculate_keep_aspect_ratio_resize(self.size(), pixmap.size())
        pixmap = pixmap.scaled(resize, spectRatioMode=Qt.KeepAspectRatio)  # 保持纵横比
        self.setPixmap(pixmap)

    def _clear_image(self):
        self.setPixmap(QPixmap())



import os.path
import re
from typing import Union

# 分卷压缩包正则
_PATTERN_7Z = r'^(.+)\.7z\.\d+$'  # test.7z.001/test.7z.002/test.7z.003
_PATTERN_RAR = r'^(.+)\.part(\d+)\.rar$'  # test.part1.rar/test.part2.rar/test.part3.rar
_PATTERN_RAR_WITHOUT_SUFFIX = r'^(.+)\.part(\d+)$'  # rar分卷文件无后缀时也能正常解压，test.part1/test.part2/test.part3
_PATTERN_ZIP = r'^(.+)\.zip$'  # zip分卷文件的第一个分卷包一般都是.zip后缀，所以.zip后缀直接视为分卷压缩文件 test.zip
_PATTERN_ZIP_VOLUME = r'^(.+)\.z\d+$'  # test.zip/test.z01/test.z02
_PATTERN_ZIP_TYPE2 = r'^(.+)\.zip\.\d+$'  # test.zip.001/test.zip.002/test.zip.003
_PATTERN_BZ2 = r'^(.+)\.bz2\.\d+$'  # test.bz2.001/test.bz2.002/test.bz2.003
_PATTERN_GZ = r'^(.+)\.gz\.\d+$'  # test.gz.001/test.gz.002/test.gz.003
_PATTERN_TAR = r'^(.+)\.tar\.\d+$'  # test.tar.001/test.tar.002/test.tar.003
_PATTERN_WIM = r'^(.+)\.wim\.\d+$'  # test.wim.001/test.wim.002/test.wim.003
_PATTERN_XZ = r'^(.+)\.xz\.\d+$'  # test.xz.001/test.xz.002/test.xz.003
_PATTERN_JOINED = [_PATTERN_7Z, _PATTERN_RAR, _PATTERN_RAR_WITHOUT_SUFFIX, _PATTERN_ZIP, _PATTERN_ZIP_VOLUME,
                   _PATTERN_ZIP_TYPE2, _PATTERN_BZ2, _PATTERN_GZ, _PATTERN_TAR, _PATTERN_WIM, _PATTERN_XZ]


def is_volume_archive_by_filename(filename: str) -> bool:
    """通过文件名判断是否为分卷压缩文件
    :param filename: str，文件名（包含文件扩展名）"""
    for pattern in _PATTERN_JOINED:
        if re.match(pattern, filename, flags=re.I):
            return True

    return False


def guess_first_volume_archive_filename(filename: str) -> Union[str, bool]:
    """根据传入的文件名，生成分卷压缩文件的首个分卷包文件名
    :param filename: str，文件名（包含文件扩展名）
    :return: 生成的首个分卷包文件名，或未生成时False"""
    if not is_volume_archive_by_filename(filename):
        return False

    guess_filename = False

    # test.7z.001/test.7z.002/test.7z.003
    if re.match(_PATTERN_7Z, filename, flags=re.I):
        filetitle = re.match(_PATTERN_7Z, filename, flags=re.I).group(1)
        guess_filename = f'{filetitle}.7z.001'
    # test.part1.rar/test.part2.rar/test.part3.rar
    elif re.match(_PATTERN_RAR, filename, flags=re.I):
        filetitle = re.match(_PATTERN_RAR, filename, flags=re.I).group(1)
        number_length = len(re.match(_PATTERN_RAR, filename, flags=re.I).group(2))  # 处理part1.rar和part01.rar的情况
        guess_filename = f'{filetitle}.part{"1".zfill(number_length)}.rar'
    # test.part1/test.part2/test.part3
    elif re.match(_PATTERN_RAR_WITHOUT_SUFFIX, filename, flags=re.I):
        filetitle = re.match(_PATTERN_RAR_WITHOUT_SUFFIX, filename, flags=re.I).group(1)
        number_length = len(re.match(_PATTERN_RAR_WITHOUT_SUFFIX, filename, flags=re.I).group(2))
        guess_filename = f'{filetitle}.part{"1".zfill(number_length)}'
    # test.zip
    elif re.match(_PATTERN_ZIP, filename, flags=re.I):
        guess_filename = filename
    # test.zip/test.z01/test.z02
    elif re.match(_PATTERN_ZIP_VOLUME, filename, flags=re.I):
        filetitle = re.match(_PATTERN_ZIP_VOLUME, filename, flags=re.I).group(1)
        guess_filename = f'{filetitle}.zip'
    # test.zip.001/test.zip.002/test.zip.003
    elif re.match(_PATTERN_ZIP_TYPE2, filename, flags=re.I):
        filetitle = re.match(_PATTERN_ZIP_TYPE2, filename, flags=re.I).group(1)
        guess_filename = f'{filetitle}.zip.001'
    # test.bz2.001/test.bz2.002/test.bz2.003
    elif re.match(_PATTERN_BZ2, filename, flags=re.I):
        filetitle = re.match(_PATTERN_BZ2, filename, flags=re.I).group(1)
        guess_filename = f'{filetitle}.bz2.001'
    # test.gz.001/test.gz.002/test.gz.003
    elif re.match(_PATTERN_GZ, filename, flags=re.I):
        filetitle = re.match(_PATTERN_GZ, filename, flags=re.I).group(1)
        guess_filename = f'{filetitle}.gz.001'
    # test.tar.001/test.tar.002/test.tar.003
    elif re.match(_PATTERN_TAR, filename, flags=re.I):
        filetitle = re.match(_PATTERN_TAR, filename, flags=re.I).group(1)
        guess_filename = f'{filetitle}.tar.001'
    # test.wim.001/test.wim.002/test.wim.003
    elif re.match(_PATTERN_WIM, filename, flags=re.I):
        filetitle = re.match(_PATTERN_WIM, filename, flags=re.I).group(1)
        guess_filename = f'{filetitle}.wim.001'
    # test.xz.001/test.xz.002/test.xz.003
    elif re.match(_PATTERN_XZ, filename, flags=re.I):
        filetitle = re.match(_PATTERN_XZ, filename, flags=re.I).group(1)
        guess_filename = f'{filetitle}.xz.001'

    return guess_filename


def get_filetitle(filename: str) -> str:
    """根据传入的文件名，提取其不含文件扩展名的文件标题
    :param filename: str，文件名（包含文件扩展名）
    :return: 不含文件扩展名的文件标题"""
    if not is_volume_archive_by_filename(filename):
        return os.path.splitext(filename)[0]

    for pattern in _PATTERN_JOINED:
        if re.match(pattern, filename, flags=re.I):
            filetitle = re.match(pattern, filename, flags=re.I).group(1)
            if filetitle:
                return filetitle

    return os.path.basename(filename)  # 兜底



import setuptools

setuptools.setup(
    name="lzytools",  # 项目名称
    version="0.0.36",  # 版本号
    author="PPJUST",  # 作者
    description="Python自用包",  # 描述
    long_description='Python自用包',  # 长描述
    long_description_content_type="text/markdown",  # 长描述语法 markdown
    url="https://github.com/PPJUST/lzytools",  # 项目地址
    packages=setuptools.find_packages(),
    python_requires='>=3',  # Python版本限制
    # setup_requires=["filetype",
    #                   "ImageHash",
    #                   "numpy<2.0.0",
    #                   "opencv_python",
    #                   "Pillow",
    #                   "pynput",
    #                   "PySide6",
    #                   "pywin32",
    #                   "rarfile",
    #                   "Send2Trash"],
)



import io
import os
from typing import Union

import numpy
from PIL import Image


def save_bytes_image(bytes_image: bytes, dirpath: str, filename: str) -> str:
    """将一个bytes图片对象保存至本地
    :param bytes_image: bytes，bytes图片对象
    :param dirpath: str，保存至该目录下
    :param filename: str，保存的文件名（不含文件扩展名）
    :return: str，保存的本地图片路径"""
    image = Image.open(io.BytesIO(bytes_image))

    # 转换图像模式，防止报错OSError: cannot write mode P as JPEG
    image = image.convert('RGB')

    # 提取文件后缀
    file_extension = image.format  # 备忘录 找一个读取bytes图片后缀的方法

    # 组合保存路径
    save_path = os.path.normpath(os.path.join(dirpath, filename + file_extension))

    # 保存到本地
    if not os.path.exists(dirpath):
        os.mkdir(save_path)
    image.save(save_path)

    return save_path


def save_numpy_image(numpy_image: numpy.ndarray, dirpath: str, filename: str) -> str:
    """将一个numpy图片对象保存至本地
    :param numpy_image: numpy.ndarray，numpy图片对象
    :param dirpath: str，保存至该目录下
    :param filename: str，保存的文件名（不含文件扩展名）
    :return: str，保存的本地图片路径"""
    # 转换为uint8类型
    numpy_image = numpy_image.astype(numpy.uint8)

    # 转换为Pillow图像对象
    image = Image.fromarray(numpy_image)

    # 提取文件后缀
    file_extension = image.format

    # 组合保存路径
    save_path = os.path.normpath(os.path.join(dirpath, filename + file_extension))

    # 保存到本地
    if not os.path.exists(dirpath):
        os.mkdir(save_path)
    image.save(save_path)

    return save_path


def save_local(image: Union[bytes, numpy.ndarray], dirpath: str, filename: str) -> str:
    """将一个bytes/numpy图片对象保存至本地
    :param image: bytes/numpy图片对象
    :param dirpath: str，保存至该目录下
    :param filename: str，保存的文件名（不含文件扩展名）
    :return: str，保存的本地图片路径"""
    if isinstance(image, bytes):
        return save_bytes_image(image, dirpath, filename)
    elif isinstance(image, numpy.ndarray):
        return save_numpy_image(image, dirpath, filename)
    else:
        raise Exception(f'不支持的图片类型：{type(image)}')



import os

import natsort


def split_path(path: str):
    """拆分路径为父目录路径，文件名（不含文件扩展名），文件扩展名
    :param path: str，需要拆分的路径
    :return: 父目录路径，文件名（不含文件扩展名），文件扩展名"""
    if os.path.isfile(path):
        _temp_path, file_extension = os.path.splitext(path)
        parent_dirpath, filetitle = os.path.split(_temp_path)
    elif os.path.isdir(path):
        file_extension = ''
        parent_dirpath, filetitle = os.path.split(path)
    else:
        raise Exception('非法路径')

    return parent_dirpath, filetitle, file_extension


def reverse_path(path: str) -> str:
    """反转路径字符串，从后往前排列目录层级
    :param path: str，路径
    :return: str，重组后的路径字符串
    """
    path = os.path.normpath(path)
    _split_path = path.split('\\')
    path_reversed = ' \\ '.join(_split_path[::-1])
    path_reversed = os.path.normpath(path_reversed)
    return path_reversed


def remove_subpaths(paths: list):
    """剔除传入路径中的子路径"""
    paths = [os.path.normpath(path) for path in paths]
    paths = natsort.os_sorted(paths)  # 排序路径
    subpaths = []
    for path in paths:
        paths_split_parents = get_parent_dirpaths(path)
        # 判断其父路径是否与原始列表存在交集，如果存在则说明其是子路径
        _s1 = set(paths)
        _s2 = set(paths_split_parents)
        if _s1 & _s2:
            subpaths.append(path)

    return [i for i in paths if i not in subpaths]


def get_parent_dirpaths(path: str) -> list:
    """获取一个路径的所有上级目录路径
    :param path: str，文件/文件夹路径
    :return: list，所有上级目录列表，层级高的在前面"""
    parent_dirs = []

    while True:
        parent_dirpath, filename = os.path.split(path)
        if filename:
            parent_dirs.append(parent_dirpath)
        else:
            break

        path = parent_dirpath

    # 反转列表顺序，使得越上级目录排在越前面
    parent_dirs = parent_dirs[::-1]

    return parent_dirs



import ctypes


def check_mutex(mutex_name: str):
    """使用互斥体检查是否已经打开了一个程序实例
    :param mutex_name: str，互斥体名称，建议使用程序名称"""
    # 创建互斥体
    mutex = ctypes.windll.kernel32.CreateMutexW(None, False, mutex_name)
    # 如果创建时报错，则说明已经创建过该互斥体，即已经有一个程序在运行了
    if ctypes.windll.kernel32.GetLastError() == 183:
        ctypes.windll.kernel32.CloseHandle(mutex)
        return True
    return False



from PySide6.QtCore import *
from PySide6.QtGui import *
from PySide6.QtWidgets import *


class LabelImageAutoSize(QLabel):
    """自适应大小显示图片"""

    def __init__(self, image_path: str = None):
        """:param image_path: str，图片路径"""
        super().__init__()
        self.setAlignment(Qt.AlignCenter)
        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        self.pixmap_original = None
        if image_path:
            self.pixmap_original = QPixmap(image_path)

    def set_image(self, image_path: str = None):
        """设置图片"""
        self.pixmap_original = QPixmap(image_path)
        self.update_image_size()

    def update_image_size(self):
        """更新图片尺寸"""
        if self.pixmap_original and not self.pixmap_original.isNull():
            scaled_pixmap = self.pixmap_original.scaled(self.size(), Qt.KeepAspectRatio, Qt.SmoothTransformation)
            self.setPixmap(scaled_pixmap)

    def resizeEvent(self, event):
        self.update_image_size()
        super().resizeEvent(event)


class LabelHiddenOverLengthText(QLabel):
    """文本框控件，自动隐藏长文本（abcd->a...）"""

    def __init__(self, parent=None):
        super().__init__(parent)
        self.text_origin = ''

    def setText(self, text):
        super().setText(text)
        self.text_origin = text
        self.check_over_length_text()

    def check_over_length_text(self):
        """检查文本是否超限"""
        # 获取字体度量
        font_metrics = self.fontMetrics()
        label_width = self.width()

        # 检查文本是否超出label宽度
        if font_metrics.horizontalAdvance(self.text_origin) > label_width:
            elided_text = font_metrics.elidedText(self.text_origin, Qt.ElideRight, label_width)
            self.setText(elided_text)

    def resizeEvent(self, event):
        super().resizeEvent(event)
        self.check_over_length_text()


class LabelHoverInfo(QLabel):
    """自动隐藏的悬浮在控件左下角的label（用于显示提示信息）"""

    _instance = None
    _is_init = False

    def __new__(cls, *args, **kwargs):
        if cls._instance is None:
            cls._instance = super().__new__(cls)
        return cls._instance

    def __init__(self, parent=None):
        if not self._is_init:
            super().__init__(parent)
            self._is_init = True

            self.setMouseTracking(True)
            self.setStyleSheet("color: blue;")
            self.setWordWrap(True)
            self.hide()
            self.raise_()  # 置顶

            self.duration = 1  # 显示时长，秒
            self.position = 'LB'  # 显示位置，RT/RB/LT/LB

            # 设置定时器
            self.timer_hidden = QTimer()
            self.timer_hidden.setSingleShot(True)
            self.timer_hidden.timeout.connect(self.hide)

    def set_duration(self, duration: float):
        """设置显示时长，秒"""
        self.duration = duration

    def set_position(self, position: str):
        """设置显示位置
        :param position: str，RT/RB/LT/LB"""
        if position.upper() not in ['RT', 'RB', 'LT', 'LB']:
            raise Exception('参数错误，请选择RT/RB/LT/LB')
        self.position = position.upper()

    def _show(self, text: str):
        """显示信息"""
        self.setText(text)
        self.reset_position()
        self.show()
        self.timer_hidden.start(self.duration * 1000)

    def reset_position(self):
        """重设坐标轴位置"""
        width, height = self.sizeHint()
        parent_width, parent_height = self.parent().size()

        if self.position == 'LT':  # LT左上角
            self.setGeometry(0, 0, width, height)
        elif self.position == 'LB':  # LB左下角
            self.setGeometry(0, parent_height - height, width, height)
        elif self.position == 'RT':  # RT右上角
            self.setGeometry(parent_width - width, 0, width, height)
        elif self.position == 'RB':  # RB右下角
            self.setGeometry(parent_width - width, parent_height - height, width, height)



"""
修复pynput模块中HotKey类的问题，使其能够正确识别小键盘数字（bug为小键盘数字的'_scan'属性没有被正确地赋值）
"""
from pynput import keyboard


class HotKeyFix(keyboard.HotKey):
    def __init__(self, keys, on_activate):
        super().__init__(keys, on_activate)

    def press(self, key):
        setattr(key, '_scan', None)
        super().press(key)

    def release(self, key):
        setattr(key, '_scan', None)
        super().release(key)


class GlobalHotKeysFix(keyboard.Listener):
    def __init__(self, hotkeys, *args, **kwargs):
        self._hotkeys = [
            HotKeyFix(HotKeyFix.parse(key), value)
            for key, value in hotkeys.items()]
        super(GlobalHotKeysFix, self).__init__(
            on_press=self._on_press,
            on_release=self._on_release,
            *args,
            **kwargs)

    def _on_press(self, key):
        for hotkey in self._hotkeys:
            hotkey.press(self.canonical(key))

    def _on_release(self, key):
        for hotkey in self._hotkeys:
            hotkey.release(self.canonical(key))


"""
下面方法可行，但不优美
class HotKeyFix(keyboard.HotKey):
    def __init__(self, keys, on_activate):
        super().__init__(keys, on_activate)

    def press(self, key):
        keys_str = [str(i) for i in self._keys]
        if str(key) in keys_str and key not in self._state:
            self._state.add(key)
            if [str(i) for i in self._state] == [str(i) for i in self._keys]:
                self._on_activate()
"""
